@@ -77,9 +77,9 @@
  *
  *      XXX - This function invokes access(), which uses the real uid,
  *      not the effective uid, so it probably does not do what you 
- *      expect.  Instead it should use FileAttributes(), which calls
- *      Posix_Stat() and uses the effective uid, but it's too risky 
- *      to fix right now.  See PR 459242.
+ *      expect.  Instead it should use Posix_EuidAccess(), which 
+ *      uses the effective uid, but it's too risky to fix right now.
+ *      See PR 459242.
  *
  * Results:
  *      TRUE    file is accessible with the process' real uid
@@ -1086,6 +1086,46 @@ File_StripSlashes(ConstUnicode path)  // IN:
 }
 
 
+/*
+ *----------------------------------------------------------------------------
+ *
+ * FileMakeTempExCreateNameFunc --
+ *
+ *      This is a helper function designed for File_MakeTempEx function.
+ *      Everytime this function is called, this creates a fileName with the
+ *      format <num><fileName> and returns back to the caller.
+ *
+ *      'num' specifies the nth time this function is called.
+ *
+ *      'data' specifies the payload that is specified when File_MakeTempEx2()
+ *      function is called. This points to a Unicode string.
+ *
+ * Results:
+ *      if successful, a dynamically allocated string with the basename of
+ *      the temp file. NULL otherwise.
+ *
+ * Side effects:
+ *      None
+ *
+ *----------------------------------------------------------------------------
+ */
+
+static Unicode
+FileMakeTempExCreateNameFunc(int num,               // IN:
+                             void *data)            // IN:
+{
+   Unicode filePath;
+
+   if (data == NULL) {
+      return NULL;
+   }
+
+   filePath = Unicode_Format("%s%d", (Unicode) data, num);
+
+   return filePath;
+}
+
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1115,14 +1155,63 @@ File_MakeTempEx(ConstUnicode dir,       // IN:
                 ConstUnicode fileName,  // IN:
                 Unicode *presult)       // OUT:
 {
-   int fd;
+   return File_MakeTempEx2(dir,
+                           TRUE,
+                           FileMakeTempExCreateNameFunc,
+                           (void *) fileName,
+                           presult);
+
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ *  File_MakeTempEx2 --
+ *
+ *      Create a temporary file or a directory.
+ *      If a temporary file is created successfully, then return an open file
+ *      descriptor to that file.
+ *
+ *      'dir' specifies the directory in which to create the file. It
+ *      must not end in a slash.
+ *
+ *      'createTempFile', if TRUE, then a temporary file will be created. If
+ *      FALSE, then a temporary directory will be created.
+ *
+ *      'createNameFunc' specifies the user-specified callback function that
+ *      will be called to construct the fileName. 'createNameFuncData' will be
+ *      passed everytime 'createNameFunc' is called. 'createNameFunc'
+ *      should return the proper fileName.
+ *
+ *      Check the documentation for File_MakeTempHelperFunc.
+ *
+ * Results:
+ *      if a temporary file is created, then Open file descriptor or -1;
+ *      if a temporary directory is created, then 0 or -1;
+ *      If successful then presult points to a dynamically allocated
+ *      string with the pathname of the temp file.
+ *
+ * Side effects:
+ *      Creates a file if successful. Errno is set on error
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+File_MakeTempEx2(ConstUnicode dir,                                // IN:
+                 Bool createTempFile,                             // IN:
+                 File_MakeTempCreateNameFunc *createNameFunc,     // IN:
+                 void *createNameFuncData,                        // IN:
+                 Unicode *presult)                                // OUT:
+{
+   int fd = -1;
    int err;
    uint32 var;
 
    Unicode path = NULL;
-   Unicode basePath = NULL;
 
-   if ((dir == NULL) || (fileName == NULL)) {
+   if ((dir == NULL) || (createNameFunc == NULL)) {
       errno = EFAULT;
       return -1;
    }
@@ -1131,21 +1220,32 @@ File_MakeTempEx(ConstUnicode dir,       // IN:
 
    *presult = NULL;
 
-   /* construct base full pathname to use */
-   basePath = Unicode_Join(dir, DIRSEPS, fileName, NULL);
-
    for (var = 0; var < 0xFFFFFFFF; var++) {
-      Unicode temp;
+      Unicode fileName = NULL;
 
       /* construct suffixed pathname to use */
       Unicode_Free(path);
+      path = NULL;
 
-      temp = Unicode_Format("%d", var);
-      ASSERT_MEM_ALLOC(temp);
-      path = Unicode_Append(basePath, temp);
-      Unicode_Free(temp);
+      fileName = (*createNameFunc)(var, createNameFuncData);
 
-      fd = Posix_Open(path, O_CREAT | O_EXCL | O_BINARY | O_RDWR, 0600);
+      if (fileName == NULL) {
+         Msg_Append(MSGID(file.maketemp.helperFuncFailed)
+                  "Failed to construct the file name\n");
+         errno = EFAULT;
+         goto exit;
+      }
+
+      /* construct base full pathname to use */
+      path = Unicode_Join(dir, DIRSEPS,  fileName, NULL);
+
+      Unicode_Free(fileName);
+
+      if (createTempFile) {
+         fd = Posix_Open(path, O_CREAT | O_EXCL | O_BINARY | O_RDWR, 0600);
+      } else {
+         fd = Posix_Mkdir(path, 0600);
+      }
 
       if (fd != -1) {
          *presult = path;
@@ -1153,7 +1253,16 @@ File_MakeTempEx(ConstUnicode dir,       // IN:
          break;
       }
 
+#if defined(_WIN32)
+      /*
+       * On windows, Posix_Open() fails with EACCES if a directory
+       * already exists with the same name. In such case, we need to
+       * ignore EACCES error and continue trying out.
+       */
+      if ((errno != EEXIST) && (errno != EACCES)) {
+#else
       if (errno != EEXIST) {
+#endif
          err = errno;
          Msg_Append(MSGID(file.maketemp.openFailed)
                  "Failed to create temporary file \"%s\": %s.\n",
@@ -1173,7 +1282,6 @@ File_MakeTempEx(ConstUnicode dir,       // IN:
 
   exit:
    err = errno;
-   Unicode_Free(basePath);
    Unicode_Free(path);
    errno = err;
 
@@ -1941,15 +2049,15 @@ File_DeleteDirectoryTree(ConstUnicode pathName)  // IN: directory to delete
    Unicode *fileList = NULL;
    Bool sawFileError = FALSE;
 
-   switch (FileAttributes(pathName, NULL)) {
-   case 0:
-      break;
-   case ENOENT:
-   case ENOTDIR:
-      /* path does not exist or is inaccessible */
-      return TRUE;
-   default:
-      return FALSE;
+   if (Posix_EuidAccess(pathName, F_OK)) {
+      switch (errno) {
+      case ENOENT:
+      case ENOTDIR:
+         /* path does not exist or is inaccessible */
+         return TRUE;
+      default:
+         break;
+      }
    }
 
    /* get list of files in current directory */
@@ -2130,7 +2238,10 @@ File_FindFileInSearchPath(const char *fileIn,      // IN:
       cur = Str_SafeAsprintf(NULL, "%s%s%s", cwd, DIRSEPS, fileIn);
    }
 
-   if (FileAttributes(cur, NULL) == 0) {
+   if (Posix_EuidAccess(cur, F_OK) == 0) {
+      goto done;
+   }
+   if (errno == ENOSYS && FileAttributes(cur, NULL) == 0) {
       goto done;
    }
 
@@ -2162,7 +2273,10 @@ File_FindFileInSearchPath(const char *fileIn,      // IN:
          }
       }
 
-      if (FileAttributes(cur, NULL) == 0) {
+      if (Posix_EuidAccess(cur, F_OK) == 0) {
+         break;
+      }
+      if (errno == ENOSYS && FileAttributes(cur, NULL) == 0) {
          break;
       }
 
