@@ -327,7 +327,7 @@ FileIO_Unlock(FileIODescriptor *file)  // IN/OUT:
    if (file->lockToken != NULL) {
       int err;
 
-      err = FileLock_Unlock(file->fileName, file->lockToken);
+      err = FileLock_Unlock(file->lockToken);
 
       if (err != 0) {
          Warning(LGPFX" %s on '%s' failed: %s\n",
@@ -373,7 +373,7 @@ FileIO_Filename(FileIODescriptor *fd)  // IN:
 }
 
 
-#if defined(_WIN32) || defined(GLIBC_VERSION_21) || defined(__APPLE__) || \
+#if defined(_WIN32) || defined(__linux__) || defined(__APPLE__) || \
     defined(__FreeBSD__)
 /*
  *----------------------------------------------------------------------
