@@ -243,14 +243,13 @@ EXTERN int FileLockWriteFile(FILELOCK_FILE_HANDLE handle,
                              uint32 requestedBytes,
                              uint32 *resultantBytes);
 
-EXTERN void *FileLockIntrinsic(ConstUnicode filePathName,
-                               Bool exclusivity,
-                               uint32 msecMaxWaitTime,
-                               const char *payload,
-                               int *err);
+EXTERN FileLockToken *FileLockIntrinsic(ConstUnicode filePathName,
+                                        Bool exclusivity,
+                                        uint32 msecMaxWaitTime,
+                                        const char *payload,
+                                        int *err);
 
-EXTERN int FileUnlockIntrinsic(ConstUnicode filePathName,
-                               const void *lockToken);
+EXTERN int FileUnlockIntrinsic(FileLockToken *tokenPtr);
 
 EXTERN Bool FileLockIsLocked(ConstUnicode filePath,
                              int *err);
