@@ -52,6 +52,7 @@
 #include "fileInt.h"
 #include "random.h"
 #include "vm_atomic.h"
+#include "util.h"
 
 #include "unicodeOperations.h"
 
@@ -75,6 +76,20 @@ typedef struct parse_table
    void *valuePtr;
 } ParseTable;
 
+/*
+ * The lock token. This is returned by the lock operation and must be sent
+ * to the unlock operation.
+ */
+
+#define FILELOCK_TOKEN_SIGNATURE 0x4B434C46  // 'FLCK' in memory
+
+struct FileLockToken
+{
+   uint32  signature;
+   Unicode pathName;
+   Unicode lockFilePath;  // &implicitReadToken for implicit read locks
+};
+
 
 /*
  *-----------------------------------------------------------------------------
@@ -919,55 +934,58 @@ Scanner(ConstUnicode lockDir,    // IN:
  */
 
 int
-FileUnlockIntrinsic(ConstUnicode pathName,  // IN:
-                    const void *lockToken)  // IN:
+FileUnlockIntrinsic(FileLockToken *tokenPtr)  // IN:
 {
    int err;
 
-   ASSERT(pathName);
-   ASSERT(lockToken);
+   ASSERT(tokenPtr && (tokenPtr->signature == FILELOCK_TOKEN_SIGNATURE));
 
-   LOG(1, ("Requesting unlock on %s\n", UTF8(pathName)));
+   LOG(1, ("Requesting unlock on %s\n", UTF8(tokenPtr->pathName)));
 
-   if (lockToken == &implicitReadToken) {
-      /*
-       * The lock token is the fixed-address implicit read lock token.
-       * Since no lock file was created no further action is required.
-       */
+   /*
+    * If the lockFilePath (a pointer) is the fixed-address token representing
+    * an implicit read lock, there is no lock file and the token can simply
+    * be discarded.
+    */
 
+   if (tokenPtr->lockFilePath == &implicitReadToken) {
       err = 0;
+
+      free(tokenPtr->pathName);
    } else {
       Unicode lockDir;
 
       /* The lock directory path */
-      lockDir = Unicode_Append(pathName, FILELOCK_SUFFIX);
+      lockDir = Unicode_Append(tokenPtr->pathName, FILELOCK_SUFFIX);
 
       /*
-       * The lock token is the (unicode) path of the lock file.
-       *
        * TODO: under vmx86_debug validate the contents of the lock file as
        *       matching the machineID and executionID.
        */
 
-      err = FileDeletionRobust((Unicode) lockToken, FALSE);
+      err = FileDeletionRobust(tokenPtr->lockFilePath, FALSE);
 
       if (err && vmx86_debug) {
-         Log(LGPFX" %s failed for '%s': %s\n",
-             __FUNCTION__, (char *) lockToken, Err_Errno2String(err));
+         Log(LGPFX" %s failed for '%s': %s\n", __FUNCTION__,
+             UTF8(tokenPtr->lockFilePath), Err_Errno2String(err));
       }
 
       /*
-       * The lockToken (a unicode path) was allocated in FileLockIntrinsic
-       * and returned to the caller hidden behind a "void *" pointer.
+       * Attempt to clean up the locking directory.
        */
 
-      Unicode_Free((Unicode) lockToken);
-
       FileRemoveDirectoryRobust(lockDir); // just in case we can clean up
 
       Unicode_Free(lockDir);
+      Unicode_Free(tokenPtr->pathName);
+      Unicode_Free(tokenPtr->lockFilePath);
    }
 
+   tokenPtr->signature = 0;        // Just in case...
+   tokenPtr->pathName = NULL;      // Just in case...
+   tokenPtr->lockFilePath = NULL;  // Just in case...
+   free(tokenPtr);
+
    return err;
 }
 
@@ -1446,7 +1464,7 @@ CreateMemberFile(FILELOCK_FILE_HANDLE entryHandle,  // IN:
  *-----------------------------------------------------------------------------
  */
 
-void *
+FileLockToken *
 FileLockIntrinsic(ConstUnicode pathName,   // IN:
                   Bool exclusivity,        // IN:
                   uint32 msecMaxWaitTime,  // IN:
@@ -1456,6 +1474,7 @@ FileLockIntrinsic(ConstUnicode pathName,   // IN:
    FILELOCK_FILE_HANDLE handle;
    LockValues myValues;
    int createFlags;
+   FileLockToken *tokenPtr;
 
    Unicode lockDir = NULL;
    Unicode entryFilePath = NULL;
@@ -1593,16 +1612,22 @@ bail:
    free(myValues.locationChecksum);
    free(myValues.executionID);
 
-   if (*err != 0) {
+   if (*err == 0) {
+      tokenPtr = Util_SafeMalloc(sizeof(FileLockToken));
+
+      tokenPtr->signature = FILELOCK_TOKEN_SIGNATURE;
+      tokenPtr->pathName = Unicode_Duplicate(pathName);
+      tokenPtr->lockFilePath = memberFilePath;
+   } else {
       Unicode_Free(memberFilePath);
-      memberFilePath = NULL;
+      tokenPtr = NULL;
 
       if (*err == EAGAIN) {
          *err = 0; // lock not acquired
       }
    }
 
-   return (void *) memberFilePath;
+   return tokenPtr;
 }
 
 
@@ -1675,6 +1700,33 @@ bail:
 }
 
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * FileLock_TokenPathName --
+ *
+ *      Return the path name associated with a lock (token). The path name
+ *      is returned as a dynamically allocated string the caller is
+ *      responsible for.
+ *
+ * Results:
+ *      As above
+ *
+ * Side effects:
+ *      None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+Unicode
+FileLock_TokenPathName(const FileLockToken *lockToken)  // IN:
+{
+   ASSERT(lockToken && (lockToken->signature == FILELOCK_TOKEN_SIGNATURE));
+
+   return Unicode_Duplicate(lockToken->pathName);
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
