@@ -41,9 +41,6 @@
 # endif
 #include <signal.h>
 #endif
-#if defined(GLIBC_VERSION_24)
-#define _GNU_SOURCE
-#endif
 #include <unistd.h>
 #include <errno.h>
 #include <sys/stat.h>
@@ -292,20 +289,6 @@ FileAttributes(ConstUnicode pathName,  // IN:
    int err;
    struct stat statbuf;
 
-#if defined(GLIBC_VERSION_24)
-   char *path;
-
-   if (fileData == NULL) {
-      if (!PosixConvertToCurrent(pathName, &path)) {
-         return -1;
-      }
-      ret = euidaccess(path, F_OK);
-      free(path);
-
-      return ret;
-   }
-#endif
-
    if (Posix_Stat(pathName, &statbuf) == -1) {
       err = errno;
    } else {
@@ -1738,7 +1721,7 @@ File_IsSameFile(ConstUnicode path1,  // IN:
 
    /*
     * First take care of the easy checks.  If the paths are identical, or if
-    * the inode numbers don't match, we're done.
+    * the inode numbers or resident devices don't match, we're done.
     */
 
    if (Unicode_Compare(path1, path2) == 0) {
@@ -1757,6 +1740,20 @@ File_IsSameFile(ConstUnicode path1,  // IN:
       return FALSE;
    }
 
+   if (st1.st_dev != st2.st_dev) {
+      return FALSE;
+   }
+
+   if (HostType_OSIsPureVMK()) {
+      /*
+       * On ESX, post change 1074635 the st_dev field of the stat structure
+       * is valid and differentiates between resident devices or NFS file
+       * systems - no need to use statfs to obtain file system information.
+       */
+
+      return TRUE;
+   }
+
    if (Posix_Statfs(path1, &stfs1) != 0) {
       return FALSE;
    }
@@ -1767,12 +1764,12 @@ File_IsSameFile(ConstUnicode path1,  // IN:
 
 #if defined(__APPLE__)
    if ((stfs1.f_flags & MNT_LOCAL) && (stfs2.f_flags & MNT_LOCAL)) {
-      return st1.st_dev == st2.st_dev;
+      return TRUE;
    }
 #else
    if ((stfs1.f_type != NFS_SUPER_MAGIC) &&
        (stfs2.f_type != NFS_SUPER_MAGIC)) {
-      return st1.st_dev == st2.st_dev;
+      return TRUE;
    }
 #endif
 
@@ -1790,8 +1787,7 @@ File_IsSameFile(ConstUnicode path1,  // IN:
     * false positive.
     */
 
-   if ((st1.st_dev == st2.st_dev) &&
-       (st1.st_mode == st2.st_mode) &&
+   if ((st1.st_mode == st2.st_mode) &&
        (st1.st_nlink == st2.st_nlink) &&
        (st1.st_uid == st2.st_uid) &&
        (st1.st_gid == st2.st_gid) &&
