@@ -47,11 +47,6 @@ extern Bool Hostinfo_GetMemoryInfoInPages(unsigned int *minSize,
 extern Bool Hostinfo_GetSwapInfoInPages(unsigned int *totalSwap,
                                         unsigned int *freeSwap);
 #endif
-#ifdef VMX86_SERVER
-extern Bool Hostinfo_GetCOSMemoryInfoInPages(unsigned int *minSize,
-                                             unsigned int *maxSize,
-                                             unsigned int *currentSize);
-#endif
 extern Bool Hostinfo_GetRatedCpuMhz(int32 cpuNumber,
                                     uint32 *mHz);
 extern char *Hostinfo_GetCpuDescription(uint32 cpuNumber);
@@ -72,6 +67,7 @@ extern Bool Hostinfo_OSIsSMP(void);
 #if defined(_WIN32)
 extern Bool Hostinfo_OSIsWinNT(void);
 extern Bool Hostinfo_OSIsWow64(void);
+DWORD Hostinfo_OpenProcessBits(void);
 #endif
 extern Bool Hostinfo_NestingSupported(void);
 extern Bool Hostinfo_TouchBackDoor(void);
