@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2010 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -16,7 +16,7 @@
  *
  *********************************************************/
 
-#ifndef _LOGFIXED_H_
+#ifndef _LOGIFXED_H_
 #define _LOGFIXED_H_
 
 #define INCLUDE_ALLOW_USERLEVEL
