@@ -27,6 +27,7 @@
 #include "vm_atomic.h"
 #include "vm_basic_types.h"
 #include "vm_basic_defs.h"
+#include "mutexRank.h"
 #include "vthreadBase.h"
 
 typedef struct MXUserExclLock   MXUserExclLock;
@@ -34,25 +35,7 @@ typedef struct MXUserRecLock    MXUserRecLock;
 typedef struct MXUserRWLock     MXUserRWLock;
 typedef struct MXUserCondVar    MXUserCondVar;
 typedef struct MXUserSemaphore  MXUserSemaphore;
-
-/*
- * Counting semaphore
- */
-
-MXUserSemaphore *MXUser_CreateSemaphore(const char *name,
-                                        MX_Rank rank);
-
-void MXUser_DestroySemaphore(MXUserSemaphore *sema);
-void MXUser_UpSemaphore(MXUserSemaphore *sema);
-void MXUser_DownSemaphore(MXUserSemaphore *sema);
-Bool MXUser_TryDownSemaphore(MXUserSemaphore *sema);
-
-Bool MXUser_TimedDownSemaphore(MXUserSemaphore *sema,
-                               uint32 msecWait);
-
-MXUserSemaphore *MXUser_CreateSingletonSemaphore(Atomic_Ptr *semaStorage,
-                                                 const char *name,
-                                                 MX_Rank rank);
+typedef struct MXUserBarrier    MXUserBarrier;
 
 /*
  * Exclusive ownership lock
@@ -92,6 +75,9 @@ Bool MXUser_TimedWaitCondVarExclLock(MXUserExclLock *lock,
 MXUserRecLock *MXUser_CreateRecLock(const char *name,
                                     MX_Rank rank);
 
+MXUserRecLock *MXUser_CreateRecLockSilent(const char *name,
+                                          MX_Rank rank);
+
 void MXUser_AcquireRecLock(MXUserRecLock *lock);
 Bool MXUser_TryAcquireRecLock(MXUserRecLock *lock);
 void MXUser_ReleaseRecLock(MXUserRecLock *lock);
@@ -138,6 +124,42 @@ MXUserRWLock *MXUser_CreateSingletonRWLock(Atomic_Ptr *lockStorage,
                                            const char *name,
                                            MX_Rank rank);
 
+/*
+ * Computational barrier
+ */
+
+MXUserBarrier *MXUser_CreateBarrier(const char *name,
+                                    MX_Rank rank,
+                                    uint32 count);
+
+void MXUser_DestroyBarrier(MXUserBarrier *barrier);
+void MXUser_EnterBarrier(MXUserBarrier *barrier);
+
+MXUserBarrier *MXUser_CreateSingletonBarrier(Atomic_Ptr *barrierStorage,
+                                             const char *name,
+                                             MX_Rank rank,
+                                             uint32 count);
+
+/*
+ * Counting semaphore
+ */
+
+MXUserSemaphore *MXUser_CreateSemaphore(const char *name,
+                                        MX_Rank rank);
+
+void MXUser_DestroySemaphore(MXUserSemaphore *sema);
+void MXUser_UpSemaphore(MXUserSemaphore *sema);
+void MXUser_DownSemaphore(MXUserSemaphore *sema);
+Bool MXUser_TryDownSemaphore(MXUserSemaphore *sema);
+
+Bool MXUser_TimedDownSemaphore(MXUserSemaphore *sema,
+                               uint32 msecWait);
+
+MXUserSemaphore *MXUser_CreateSingletonSemaphore(Atomic_Ptr *semaStorage,
+                                                 const char *name,
+                                                 MX_Rank rank);
+
+
 
 /*
  * Generic conditional variable functions.
@@ -167,11 +189,9 @@ MXUserRecLock *MXUser_InitFromMXRec(const char *name,
                                     MX_Rank rank,
                                     Bool isBelowBull);
 
-#if defined(__i386__) || defined(__x86_64__)
 #if defined(VMX86_STATS) 
 #define MXUSER_STATS  // stats "only inside the VMX" when requested
 #endif
-#endif  // X86 and X86-64
 #endif
 
 #if defined(VMX86_DEBUG)
@@ -179,7 +199,7 @@ MXUserRecLock *MXUser_InitFromMXRec(const char *name,
 #endif
 
 #if defined(MXUSER_DEBUG)
-Bool MXUser_AnyLocksHeld(VThreadID tid);
+Bool MXUser_IsCurThreadHoldingLocks(void);
 void MXUser_TryAcquireFailureControl(Bool (*func)(const char *lockName));
 #endif
 
@@ -188,9 +208,6 @@ void MXUser_StatisticsControl(double contentionRatio,
                               uint64 minCount);
 
 void MXUser_PerLockData(void);
-void MXUser_PerThreadData(VThreadID tid,
-                          uint64 *totalAcquisitions,
-                          uint64 *contendedAcquisitions);
 #endif
 
 void MXUser_SetInPanic(void);
