@@ -338,48 +338,6 @@ struct VixMsgPowerOpRequest {
 VixMsgPowerOpRequest;
 
 
-
-/*
- * **********************************************************
- * Set NIC request. The response is just a generic
- * response header (it has no body).
- */
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgNICBandwidth {
-   Bool        validNICNum;
-   int32       nicNum;
-   char        pvnGUID[64];
-
-   uint32      totalBandwidth;
-   uint32      maxSendBandwidth;
-   uint32      maxReceiveBandwidth;
-
-   uint32      packetLossPattern;
-   uint32      packetLossRate;
-   uint32      packetLossMinBurstDuration;
-   uint32      packetLossMaxBurstDuration;
-
-   uint32      minLatency;
-   uint32      maxLatency;
-
-   uint32      options;
-}
-#include "vmware_pack_end.h"
-VixMsgNICBandwidth;
-   
-
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgSetNICBandwidthRequest {
-   VixCommandRequestHeader header;
-   VixMsgNICBandwidth      nicSettings;
-}
-#include "vmware_pack_end.h"
-VixMsgSetNICBandwidthRequest;
-
-
-
 /*
  * **********************************************************
  * Get/Set Properties Request
@@ -451,6 +409,29 @@ struct VixMsgConfigurationObjectType {
 #include "vmware_pack_end.h"
 VixMsgConfigurationObjectType;
 
+typedef
+#include "vmware_pack_begin.h"
+struct VixMsgNICBandwidth {
+   Bool        validNICNum;
+   int32       nicNum;
+   char        pvnGUID[64];
+
+   uint32      totalBandwidth;
+   uint32      maxSendBandwidth;
+   uint32      maxReceiveBandwidth;
+
+   uint32      packetLossPattern;
+   uint32      packetLossRate;
+   uint32      packetLossMinBurstDuration;
+   uint32      packetLossMaxBurstDuration;
+
+   uint32      minLatency;
+   uint32      maxLatency;
+
+   uint32      options;
+}
+#include "vmware_pack_end.h"
+VixMsgNICBandwidth;
 
 typedef
 #include "vmware_pack_begin.h"
@@ -683,6 +664,20 @@ struct VixCommandRenameFileRequest {
 #include "vmware_pack_end.h"
 VixCommandRenameFileRequest;
 
+typedef
+#include "vmware_pack_begin.h"
+struct VixCommandRenameFileRequestEx {
+   VixCommandRequestHeader header;
+
+   int32                   copyFileOptions;
+   uint32                  oldPathNameLength;
+   uint32                  newPathNameLength;
+   uint32                  filePropertiesLength;
+   Bool                    overwrite;
+}
+#include "vmware_pack_end.h"
+VixCommandRenameFileRequestEx;
+
 typedef
 #include "vmware_pack_begin.h"
 struct VixCommandHgfsSendPacket {
@@ -694,6 +689,25 @@ struct VixCommandHgfsSendPacket {
 #include "vmware_pack_end.h"
 VixCommandHgfsSendPacket;
 
+typedef
+#include "vmware_pack_begin.h"
+struct VixMsgSetGuestFileAttributesRequest {
+   VixCommandRequestHeader header;
+
+   int32                  fileOptions;
+   int64                  createTime;
+   int64                  accessTime;
+   int64                  modificationTime;
+   int32                  ownerId;
+   int32                  groupId;
+   int32                  permissions;
+   Bool                   hidden;
+   Bool                   readOnly;
+   uint32                 guestPathNameLength;
+}
+#include "vmware_pack_end.h"
+VixMsgSetGuestFileAttributesRequest;
+
 
 /*
  * **********************************************************
@@ -775,6 +789,19 @@ struct VixMsgCreateFileRequest {
 #include "vmware_pack_end.h"
 VixMsgCreateFileRequest;
 
+typedef
+#include "vmware_pack_begin.h"
+struct VixMsgCreateFileRequestEx {
+   VixCommandRequestHeader header;
+
+   int32                   fileOptions;
+   uint32                  guestPathNameLength;
+   uint32                  filePropertiesLength;
+   Bool                    createParentDirectories;
+}
+#include "vmware_pack_end.h"
+VixMsgCreateFileRequestEx;
+
 
 /*
  * **********************************************************
@@ -1551,6 +1578,21 @@ struct VixMsgCreateTempFileRequest {
 VixMsgCreateTempFileRequest;
 
 
+typedef
+#include "vmware_pack_begin.h"
+struct VixMsgCreateTempFileRequestEx {
+   VixCommandRequestHeader header;
+
+   int32                   options;
+   uint32                  filePrefixLength;
+   uint32                  fileSuffixLength;
+   uint32                  directoryPathLength;
+   uint32                  propertyListLength;
+}
+#include "vmware_pack_end.h"
+VixMsgCreateTempFileRequestEx;
+
+
 /*
  * **********************************************************
  * Connect/Disconnect device request. The response is just a generic
@@ -2220,6 +2262,19 @@ enum {
 
    VIX_COMMAND_LIST_FILES                       = 177,
 
+   VIX_COMMAND_CREATE_DIRECTORY_EX              = 178,
+
+   VIX_COMMAND_MOVE_GUEST_FILE_EX               = 179,
+
+   VIX_COMMAND_MOVE_GUEST_DIRECTORY             = 180,
+
+   VIX_COMMAND_CREATE_TEMPORARY_FILE_EX         = 181,
+
+   VIX_COMMAND_CREATE_TEMPORARY_DIRECTORY       = 182,
+
+   VIX_COMMAND_SET_GUEST_FILE_ATTRIBUTES        = 183,
+
+   VIX_COMMAND_COPY_FILE_FROM_GUEST_TO_READER   = 184,
    /*
     * HOWTO: Adding a new Vix Command. Step 2a.
     *
@@ -2230,7 +2285,7 @@ enum {
     * Once a new command is added here, a command info field needs to be added
     * in bora/lib/foundryMsg. as well.
     */
-   VIX_COMMAND_LAST_NORMAL_COMMAND              = 178,
+   VIX_COMMAND_LAST_NORMAL_COMMAND              = 185,
 
    VIX_TEST_UNSUPPORTED_TOOLS_OPCODE_COMMAND    = 998,
    VIX_TEST_UNSUPPORTED_VMX_OPCODE_COMMAND      = 999,
