@@ -308,7 +308,11 @@ typedef int64 VmTimeVirtualClock;  /* Virtual Clock kept in CPU cycles */
 #elif defined __APPLE__
    /* Mac OS hosts use the same formatters for 32- and 64-bit. */
    #define FMT64 "ll"
-   #define FMTSZ "z"
+   #if KERNEL
+      #define FMTSZ "l"
+   #else
+      #define FMTSZ "z"
+   #endif
    #define FMTPD "l"
    #define FMTH ""
 #elif __GNUC__
@@ -989,11 +993,7 @@ typedef int MXSemaHandle;
  * Define type for poll device handles.
  */
 
-#ifdef _WIN32
-typedef uintptr_t PollDevHandle;
-#else
-typedef int PollDevHandle;
-#endif
+typedef int64 PollDevHandle;
 
 /*
  * Define the utf16_t type.
