@@ -997,9 +997,23 @@ typedef uint32 ToolsVersion;
 #define   TOOLS_VERSION_HOSTED10_RELEASE_V_MNR  4
 #define   TOOLS_VERSION_HOSTED10_RELEASE_V_BASE 2
 
-#define   TOOLS_VERSION_CURRENT        TOOLS_VERSION_HOSTED10_RELEASE
-#define   TOOLS_VERSION_CURRENT_STR    TOOLS_VERSION_TO_STR(TOOLS_VERSION_HOSTED10_RELEASE)
-#define   TOOLS_VERSION_CURRENT_CSV    TOOLS_VERSION_TO_CSV(TOOLS_VERSION_HOSTED10_RELEASE)
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_HOSTED10_UPDATE1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED10_UPDATE1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_HOSTED10_UPDATE1_V_MJR  8
+#define   TOOLS_VERSION_HOSTED10_UPDATE1_V_MNR  4
+#define   TOOLS_VERSION_HOSTED10_UPDATE1_V_BASE 3
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_HOSTED10_UPDATE2 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED10_UPDATE2_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_HOSTED10_UPDATE2_V_MJR  8
+#define   TOOLS_VERSION_HOSTED10_UPDATE2_V_MNR  4
+#define   TOOLS_VERSION_HOSTED10_UPDATE2_V_BASE 4
+
+#define   TOOLS_VERSION_CURRENT        TOOLS_VERSION_HOSTED10_UPDATE2
+#define   TOOLS_VERSION_CURRENT_STR    TOOLS_VERSION_TO_STR(TOOLS_VERSION_HOSTED10_UPDATE2)
+#define   TOOLS_VERSION_CURRENT_CSV    TOOLS_VERSION_TO_CSV(TOOLS_VERSION_HOSTED10_UPDATE2)
 
 /*
  * The extended Tools version is the current Tools version with the
