@@ -40,13 +40,20 @@
 /** TCLO channel name for the HGFS driver. */
 #define TOOLS_HGFS_NAME           "tools-hgfs"
 
+/*
+ * The TCLO channel names used by the View Agent Service.
+ * vdiagent may register RPC callbacks to be notified of certain events not
+ * handled by Tools Service.
+ */
+#define TOOLS_VDIAGENT_NAME       "vdiagent"
+
 /** Reply from host when the command is not recognized. */
 #define RPCI_UNKNOWN_COMMAND      "Unknown command"
 
 /*
  * Tools options.
  */
- 
+
 #define TOOLSOPTION_COPYPASTE                "copypaste"
 #define TOOLSOPTION_AUTOHIDE                 "autohide"
 #define TOOLSOPTION_BROADCASTIP              "broadcastIP"
@@ -67,6 +74,12 @@
 
 #define DISK_SHRINK_CMD             "disk.shrink"
 
+/*
+ * Auto-lock commands.
+ */
+
+#define DESKTOP_AUTOLOCK_CMD        "Autolock_Desktop"
+
 
 /*
  * The max selection buffer length has to be less than the
