@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2008-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2008 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -121,8 +121,6 @@ typedef struct RpcDebugPlugin {
    RpcDebugSendFn       sendFn;
    /** Shutdown function. */
    RpcDebugShutdownFn   shutdownFn;
-   /** Plugin data that debug plugins can also export. */
-   ToolsPluginData     *plugin;
 } RpcDebugPlugin;
 
 
