@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2010 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -22,6 +22,8 @@
 #include "userlock.h"
 #include "ulInt.h"
 
+#define MXUSER_BARRIER_SIGNATURE 0x52524142 // 'BARR' in memory
+
 struct BarrierContext
 {
    uint32           count;    // Number of threads currently in this context
@@ -34,8 +36,8 @@ struct MXUserBarrier
 {
    MXUserHeader     header;        // Barrier's ID information
    MXUserExclLock  *lock;          // Barrier's (internal) lock
-   uint32           configCount;   // Hold until this many threads arrive
-   volatile uint32  curContext;    // Arrivals go to this context
+   uint32           configCount;   // Hold until this many threads arrive.
+   uint32           curContext;    // Normal arrivals go to this context
    BarrierContext   contexts[2];   // The normal and abnormal contexts
 };
 
@@ -67,9 +69,8 @@ MXUserDumpBarrier(MXUserHeader *header)  // IN:
    Warning("\tsignature 0x%X\n", barrier->header.signature);
    Warning("\tname %s\n", barrier->header.name);
    Warning("\trank 0x%X\n", barrier->header.rank);
-   Warning("\tserial number %u\n", barrier->header.bits.serialNumber);
 
-   Warning("\tlock 0x%p\n", barrier->lock);
+   Warning("\tlock %p\n", barrier->lock);
    Warning("\tconfigured count %u\n", barrier->configCount);
    curContext = barrier->curContext;
 
@@ -109,8 +110,8 @@ MXUserDumpBarrier(MXUserHeader *header)  // IN:
  */
 
 MXUserBarrier *
-MXUser_CreateBarrier(const char *userName,  // IN: shall be known as
-                     MX_Rank rank,          // IN: internal lock's rank
+MXUser_CreateBarrier(const char *userName,  // IN:
+                     MX_Rank rank,          // IN:
                      uint32 count)          // IN:
 {
    char *properName;
@@ -118,7 +119,7 @@ MXUser_CreateBarrier(const char *userName,  // IN: shall be known as
 
    ASSERT(count);
 
-   barrier = Util_SafeCalloc(1, sizeof *barrier);
+   barrier = Util_SafeCalloc(1, sizeof(*barrier));
 
    if (userName == NULL) {
       properName = Str_SafeAsprintf(NULL, "Barrier-%p", GetReturnAddress());
@@ -154,14 +155,15 @@ MXUser_CreateBarrier(const char *userName,  // IN: shall be known as
    barrier->configCount = count;
    barrier->curContext = 0;
 
-   barrier->header.signature = MXUserGetSignature(MXUSER_TYPE_BARRIER);
    barrier->header.name = properName;
+   barrier->header.signature = MXUSER_BARRIER_SIGNATURE;
    barrier->header.rank = rank;
-   barrier->header.bits.serialNumber = MXUserAllocSerialNumber();
    barrier->header.dumpFunc = MXUserDumpBarrier;
-   barrier->header.statsFunc = NULL;
 
-   MXUserAddToList(&barrier->header);
+#if defined(MXUSER_STATS)
+   barrier->header.statsFunc = NULL;
+   barrier->header.identifier = MXUserAllocID();
+#endif
 
    return barrier;
 }
@@ -184,10 +186,10 @@ MXUser_CreateBarrier(const char *userName,  // IN: shall be known as
  */
 
 void
-MXUser_DestroyBarrier(MXUserBarrier *barrier)  // IN/OUT:
+MXUser_DestroyBarrier(MXUserBarrier *barrier)  // IN:
 {
    if (LIKELY(barrier != NULL)) {
-      MXUserValidateHeader(&barrier->header, MXUSER_TYPE_BARRIER);
+      ASSERT(barrier->header.signature == MXUSER_BARRIER_SIGNATURE);
 
       if ((barrier->contexts[0].count != 0) ||
           (barrier->contexts[1].count != 0)) {
@@ -196,15 +198,12 @@ MXUser_DestroyBarrier(MXUserBarrier *barrier)  // IN/OUT:
                             __FUNCTION__);
       }
 
-      barrier->header.signature = 0;  // just in case...
-
-      MXUserRemoveFromList(&barrier->header);
-
       MXUser_DestroyCondVar(barrier->contexts[0].condVar);
       MXUser_DestroyCondVar(barrier->contexts[1].condVar);
       MXUser_DestroyExclLock(barrier->lock);
 
-      free(barrier->header.name);
+      barrier->header.signature = 0;  // just in case...
+      free((void *) barrier->header.name);  // avoid const warnings
       barrier->header.name = NULL;
       free(barrier);
    }
@@ -222,7 +221,7 @@ MXUser_DestroyBarrier(MXUserBarrier *barrier)  // IN/OUT:
  *      threads that have entered reaches the configured number upon which
  *      time all of the threads will return from this routine.
  *
- *      "Nobody comes out until everyone goes in."
+ *      "Nobody comes out until everone goes in."
  *
  * Results:
  *      None
@@ -237,43 +236,33 @@ void
 MXUser_EnterBarrier(MXUserBarrier *barrier)  // IN/OUT:
 {
    BarrierContext *ptr;
-   uint32 context;
 
-   ASSERT(barrier);
-   MXUserValidateHeader(&barrier->header, MXUSER_TYPE_BARRIER);
+   ASSERT(barrier && (barrier->header.signature == MXUSER_BARRIER_SIGNATURE));
 
    MXUser_AcquireExclLock(barrier->lock);
 
-   context = barrier->curContext;
-   ptr = &barrier->contexts[context];
+   ptr = &barrier->contexts[barrier->curContext];
 
    ptr->count++;
 
    if (ptr->count == barrier->configCount) {
+      /* The last thread has entered; release the other threads */
       /*
-       * The last thread has entered; release the other threads
-       *
        * Flip the current context. Should a thread leave the barrier and
        * enter the barrier while the barrier is "emptying" the thread will
-       * park on the condVar that is not "emptying". Eventually everything
+       * parked on the condVar that is not "emptying". Eventually everything
        * will "work out" and all of the threads will be parked on the opposite
        * context's condVar.
        */
 
-      barrier->curContext = (context + 1) & 0x1;
+      barrier->curContext = (barrier->curContext + 1) & 0x1;
       ASSERT(barrier->contexts[barrier->curContext].count == 0);
 
       /* Wake up all of the waiting threads. */
       MXUser_BroadcastCondVar(ptr->condVar);
    } else {
-      /*
-       * Not the last thread in... wait/sleep until the last thread appears.
-       * Protect against spurious wakeups.
-       */
-
-      while (barrier->curContext == context) {
-         MXUser_WaitCondVarExclLock(barrier->lock, ptr->condVar);
-      }
+      /* Not the last thread in... sleep until the last thread appears */
+      MXUser_WaitCondVarExclLock(barrier->lock, ptr->condVar);
    }
 
    ptr->count--;
@@ -311,18 +300,19 @@ MXUser_CreateSingletonBarrier(Atomic_Ptr *barrierStorage,  // IN/OUT:
 
    ASSERT(barrierStorage);
 
-   barrier = Atomic_ReadPtr(barrierStorage);
+   barrier = (MXUserBarrier *) Atomic_ReadPtr(barrierStorage);
 
    if (UNLIKELY(barrier == NULL)) {
       MXUserBarrier *newBarrier = MXUser_CreateBarrier(name, rank, count);
 
-      barrier = Atomic_ReadIfEqualWritePtr(barrierStorage, NULL,
-                                           (void *) newBarrier);
+      barrier = (MXUserBarrier *) Atomic_ReadIfEqualWritePtr(barrierStorage,
+                                                             NULL,
+                                                           (void *) newBarrier);
 
       if (barrier) {
          MXUser_DestroyBarrier(newBarrier);
       } else {
-         barrier = Atomic_ReadPtr(barrierStorage);
+         barrier = (MXUserBarrier *) Atomic_ReadPtr(barrierStorage);
       }
    }
 
