@@ -21,6 +21,7 @@
 #endif
 
 #include "vmware.h"
+#include "vm_basic_asm.h"
 #include "str.h"
 #include "util.h"
 #include "hashTable.h"
@@ -88,6 +89,8 @@ MXUserNativeRWSupported(void)
                                     GetProcAddress(kernel32,
                                                    "ReleaseSRWLockExclusive");
 
+         COMPILER_MEM_BARRIER();
+
          result = ((pInitializeSRWLock != NULL) &&
                    (pAcquireSRWLockShared != NULL) &&
                    (pReleaseSRWLockShared != NULL) &&
@@ -232,13 +235,18 @@ typedef enum {
 } HolderState;
 
 typedef struct {
-#if defined(MXUSER_STATS)
-   VmTimeType   holdStart;
-#endif
-
    HolderState  state;
+   VmTimeType   holdStart;
 } HolderContext;
 
+typedef struct {
+   MXUserAcquisitionStats  acquisitionStats;
+   Atomic_Ptr              acquisitionHisto;
+
+   MXUserBasicStats        heldStats;
+   Atomic_Ptr              heldHisto;
+} MXUserStats;
+
 struct MXUserRWLock
 {
    MXUserHeader    header;
@@ -250,13 +258,7 @@ struct MXUserRWLock
    Atomic_uint32   holderCount;
    HashTable      *holderTable;
 
-#if defined(MXUSER_STATS)
-   MXUserAcquisitionStats  acquisitionStats;
-   Atomic_Ptr              acquisitionHisto;
-
-   MXUserBasicStats        heldStats;
-   Atomic_Ptr              heldHisto;
-#endif
+   Atomic_Ptr      statsMem;
 };
 
 
@@ -280,48 +282,52 @@ struct MXUserRWLock
 static void
 MXUserStatsActionRW(MXUserHeader *header)  // IN:
 {
-   Bool isHot;
-   Bool doLog;
-   double contentionRatio;
-
    MXUserRWLock *lock = (MXUserRWLock *) header;
+   MXUserStats *stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
 
-   /*
-    * Dump the statistics for the specified lock.
-    */
+   if (stats) {
+      Bool isHot;
+      Bool doLog;
+      double contentionRatio;
 
-   MXUserDumpAcquisitionStats(&lock->acquisitionStats, header);
+      /*
+       * Dump the statistics for the specified lock.
+       */
 
-   if (Atomic_ReadPtr(&lock->acquisitionHisto) != NULL) {
-      MXUserHistoDump(Atomic_ReadPtr(&lock->acquisitionHisto), header);
-   }
+      MXUserDumpAcquisitionStats(&stats->acquisitionStats, header);
 
-   MXUserDumpBasicStats(&lock->heldStats, header);
+      if (Atomic_ReadPtr(&stats->acquisitionHisto) != NULL) {
+         MXUserHistoDump(Atomic_ReadPtr(&stats->acquisitionHisto), header);
+      }
 
-   if (Atomic_ReadPtr(&lock->heldHisto) != NULL) {
-      MXUserHistoDump(Atomic_ReadPtr(&lock->heldHisto), header);
-   }
+      MXUserDumpBasicStats(&stats->heldStats, header);
 
-   /*
-    * Has the lock gone "hot"? If so, implement the hot actions.
-    * Allow the read and write statistics to go independently hot.
-    */
+      if (Atomic_ReadPtr(&stats->heldHisto) != NULL) {
+         MXUserHistoDump(Atomic_ReadPtr(&stats->heldHisto), header);
+      }
 
-   MXUserKitchen(&lock->acquisitionStats, &contentionRatio, &isHot, &doLog);
+      /*
+       * Has the lock gone "hot"? If so, implement the hot actions.
+       * Allow the read and write statistics to go independently hot.
+       */
 
-   if (isHot) {
-      MXUserForceHisto(&lock->acquisitionHisto,
-                       MXUSER_STAT_CLASS_ACQUISITION,
-                       MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
-                       MXUSER_DEFAULT_HISTO_DECADES);
-      MXUserForceHisto(&lock->heldHisto,
-                       MXUSER_STAT_CLASS_HELD,
-                       MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
-                       MXUSER_DEFAULT_HISTO_DECADES);
+      MXUserKitchen(&stats->acquisitionStats, &contentionRatio, &isHot,
+                    &doLog);
 
-      if (doLog) {
-         Log("HOT LOCK (%s); contention ratio %f\n",
-             lock->header.name, contentionRatio);
+      if (isHot) {
+         MXUserForceHisto(&stats->acquisitionHisto,
+                          MXUSER_STAT_CLASS_ACQUISITION,
+                          MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
+                          MXUSER_DEFAULT_HISTO_DECADES);
+         MXUserForceHisto(&stats->heldHisto,
+                          MXUSER_STAT_CLASS_HELD,
+                          MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
+                          MXUSER_DEFAULT_HISTO_DECADES);
+
+         if (doLog) {
+            Log("HOT LOCK (%s); contention ratio %f\n",
+                lock->header.name, contentionRatio);
+         }
       }
    }
 }
@@ -362,7 +368,6 @@ MXUserDumpRWLock(MXUserHeader *header)  // IN:
 
 #if defined(MXUSER_DEBUG)
       Warning("\tcaller 0x%p\n", lock->recursiveLock.ownerRetAddr);
-      Warning("\tVThreadID %d\n", (int) lock->recursiveLock.portableThreadID);
 #endif
    }
 
@@ -443,6 +448,8 @@ MXUser_CreateRWLock(const char *userName,  // IN:
    }
 
    if (LIKELY(lockInited)) {
+      MXUserStats *stats;
+
       lock->holderTable = HashTable_Alloc(256,
                                           HASH_INT_KEY | HASH_FLAG_ATOMIC,
                                           MXUserFreeHashEntry);
@@ -456,10 +463,15 @@ MXUser_CreateRWLock(const char *userName,  // IN:
       lock->header.statsFunc = MXUserStatsActionRW;
       lock->header.identifier = MXUserAllocID();
 
-      MXUserAddToList(&lock->header);
-      MXUserAcquisitionStatsSetUp(&lock->acquisitionStats);
-      MXUserBasicStatsSetUp(&lock->heldStats, MXUSER_STAT_CLASS_HELD);
+      stats = Util_SafeCalloc(1, sizeof(*stats));
+
+      MXUserAcquisitionStatsSetUp(&stats->acquisitionStats);
+      MXUserBasicStatsSetUp(&stats->heldStats, MXUSER_STAT_CLASS_HELD);
+#else
+      stats = NULL;
 #endif
+
+      MXUserAddToList(&lock->header);
    } else {
       free(properName);
       free(lock);
@@ -490,6 +502,8 @@ void
 MXUser_DestroyRWLock(MXUserRWLock *lock)  // IN:
 {
    if (LIKELY(lock != NULL)) {
+      MXUserStats *stats;
+
       ASSERT(lock->header.signature == MXUSER_RW_SIGNATURE);
 
       if (Atomic_Read(&lock->holderCount) != 0) {
@@ -513,15 +527,20 @@ MXUser_DestroyRWLock(MXUserRWLock *lock)  // IN:
          MXRecLockDestroy(&lock->recursiveLock);  
       }
 
-#if defined(MXUSER_STATS)
       MXUserRemoveFromList(&lock->header);
-      MXUserAcquisitionStatsTearDown(&lock->acquisitionStats);
-      MXUserBasicStatsTearDown(&lock->heldStats);
-      MXUserHistoTearDown(Atomic_ReadPtr(&lock->acquisitionHisto));
-      MXUserHistoTearDown(Atomic_ReadPtr(&lock->heldHisto));
-#endif
 
-      HashTable_Free(lock->holderTable);
+      stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
+
+      if (stats) {
+         MXUserAcquisitionStatsTearDown(&stats->acquisitionStats);
+         MXUserBasicStatsTearDown(&stats->heldStats);
+         MXUserHistoTearDown(Atomic_ReadPtr(&stats->acquisitionHisto));
+         MXUserHistoTearDown(Atomic_ReadPtr(&stats->heldHisto));
+
+         free(stats);
+      }
+
+      HashTable_FreeUnsafe(lock->holderTable);
       lock->header.signature = 0;  // just in case...
       free((void *) lock->header.name);  // avoid const warnings
       lock->header.name = NULL;
@@ -551,22 +570,17 @@ static HolderContext *
 MXUserGetHolderContext(MXUserRWLock *lock)  // IN:
 {
    HolderContext *result;
-   VThreadID tid = VThread_CurID();
+   void *threadID = MXUserGetNativeTID();
 
    ASSERT(lock->holderTable);
 
-   if (!HashTable_Lookup(lock->holderTable, (void *) (uintptr_t) tid,
-                         (void **) &result)) {
+   if (!HashTable_Lookup(lock->holderTable, threadID, (void **) &result)) {
       HolderContext *newContext = Util_SafeMalloc(sizeof(HolderContext));
 
-#if defined(MXUSER_STATS)
       newContext->holdStart = 0;
-#endif
-
       newContext->state = RW_UNLOCKED;
 
-      result = HashTable_LookupOrInsert(lock->holderTable,
-                                        (void *) (uintptr_t) tid,
+      result = HashTable_LookupOrInsert(lock->holderTable, threadID,
                                         (void *) newContext);
 
       if (result != newContext) {
@@ -600,14 +614,9 @@ MXUserAcquisition(MXUserRWLock *lock,  // IN/OUT:
 {
    int err = 0;
    Bool contended;
+   MXUserStats *stats;
    HolderContext *myContext;
 
-#if defined(MXUSER_STATS)
-   VmTimeType begin;
-   VmTimeType value;
-   MXUserHisto *histo;
-#endif
-
    ASSERT(lock && (lock->header.signature == MXUSER_RW_SIGNATURE));
 
    MXUserAcquisitionTracking(&lock->header, TRUE);
@@ -623,40 +632,54 @@ MXUserAcquisition(MXUserRWLock *lock,  // IN/OUT:
                                                                    "Write");
    }
 
-#if defined(MXUSER_STATS)
-   begin = Hostinfo_SystemTimerNS();
-#endif
+   stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
 
-   contended = lock->useNative ? MXUserNativeRWAcquire(&lock->nativeLock,
-                                                       forRead, &err) :
-                                 MXRecLockAcquire(&lock->recursiveLock,
-                                                  GetReturnAddress());
-   if (UNLIKELY(err != 0)) {
-      MXUserDumpAndPanic(&lock->header, "%s: Error %d: contended %d\n",
-                         __FUNCTION__, err, contended);
+   if (stats) {
+      VmTimeType begin = Hostinfo_SystemTimerNS();
+      VmTimeType value;
+      MXUserHisto *histo;
+
+      contended = lock->useNative ? MXUserNativeRWAcquire(&lock->nativeLock,
+                                                          forRead, &err) :
+                                    MXRecLockAcquire(&lock->recursiveLock,
+                                                     GetReturnAddress());
+
+      value = Hostinfo_SystemTimerNS() - begin;
+
+      if (UNLIKELY(err != 0)) {
+         MXUserDumpAndPanic(&lock->header, "%s: Error %d: contended %d\n",
+                            __FUNCTION__, err, contended);
+      }
+
+      /*
+       * The statistics are not atomically safe so protect them when
+       * necessary
+       */
+
+      if (forRead && lock->useNative) {
+         MXRecLockAcquire(&lock->recursiveLock, GetReturnAddress());
+      }
+
+      MXUserAcquisitionSample(&stats->acquisitionStats, TRUE, contended,
+                              value);
+
+      histo = Atomic_ReadPtr(&stats->acquisitionHisto);
+
+      if (UNLIKELY(histo != NULL)) {
+         MXUserHistoSample(histo, value);
+      }
+
+      if (forRead && lock->useNative) {
+         MXRecLockRelease(&lock->recursiveLock);
+      }
+   } else {
+      if (LIKELY(lock->useNative)) {
+         MXUserNativeRWAcquire(&lock->nativeLock, forRead, &err);
+      } else {
+         MXRecLockAcquire(&lock->recursiveLock, GetReturnAddress());
+      }
    }
 
-#if defined(MXUSER_STATS)
-   value = Hostinfo_SystemTimerNS() - begin;
-
-   /* The statistics are not atomically safe so protect them when necessary */
-   if (forRead && lock->useNative) {
-      MXRecLockAcquire(&lock->recursiveLock, GetReturnAddress());
-   }
-
-   MXUserAcquisitionSample(&lock->acquisitionStats, contended, value);
-
-   histo = Atomic_ReadPtr(&lock->acquisitionHisto);
-
-   if (UNLIKELY(histo != NULL)) {
-      MXUserHistoSample(histo, value);
-   }
-
-   if (forRead && lock->useNative) {
-      MXRecLockRelease(&lock->recursiveLock);
-   }
-#endif
-
    if (!forRead || !lock->useNative) {
       ASSERT(Atomic_Read(&lock->holderCount) == 0);
    }
@@ -664,9 +687,9 @@ MXUserAcquisition(MXUserRWLock *lock,  // IN/OUT:
    Atomic_Inc(&lock->holderCount);
    myContext->state = forRead ? RW_LOCKED_FOR_READ : RW_LOCKED_FOR_WRITE;
 
-#if defined(MXUSER_STATS)
-   myContext->holdStart = Hostinfo_SystemTimerNS();
-#endif
+   if (stats) {
+      myContext->holdStart = Hostinfo_SystemTimerNS();
+   }
 }
 
 
@@ -675,7 +698,9 @@ MXUserAcquisition(MXUserRWLock *lock,  // IN/OUT:
  *
  * MXUser_AcquireForRead --
  *
- *      Acquire a read-write lock for read-shared access.
+ *      Acquire a read-write lock for read-shared access. A thread may have
+ *      only one read lock outstanding on a read-write lock - no recursive
+ *      access.
  *
  * Results:
  *      The lock is acquired.
@@ -698,7 +723,8 @@ MXUser_AcquireForRead(MXUserRWLock *lock)  // IN/OUT:
  *
  * MXUser_AcquireForWrite --
  *
- *      Acquire a read-write lock for write-exclusive access.
+ *      Acquire a read-write lock for write-exclusive access. A thread may
+ *      have only a single write lock outstanding on a read-write lock.
  *
  * Results:
  *      The lock is acquired.
@@ -782,48 +808,50 @@ MXUser_IsCurThreadHoldingRWLock(MXUserRWLock *lock,  // IN:
 void
 MXUser_ReleaseRWLock(MXUserRWLock *lock)  // IN/OUT:
 {
+   MXUserStats *stats;
    HolderContext *myContext;
 
-#if defined(MXUSER_STATS)
-   VmTimeType holdEnd = Hostinfo_SystemTimerNS();
-   VmTimeType duration;
-   MXUserHisto *histo;
-#endif
-
    ASSERT(lock && (lock->header.signature == MXUSER_RW_SIGNATURE));
 
    myContext = MXUserGetHolderContext(lock);
+   stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
 
-   if (UNLIKELY(myContext->state == RW_UNLOCKED)) {
-      uint32 lockCount = Atomic_Read(&lock->holderCount);
+   if (stats) {
+      VmTimeType duration = Hostinfo_SystemTimerNS() - myContext->holdStart;
+      MXUserHisto *histo;
 
-      MXUserDumpAndPanic(&lock->header,
-                         "%s: Non-owner release of an %s read-write lock\n",
-                         __FUNCTION__,
-                         lockCount == 0 ? "unacquired" : "acquired");
+      if (UNLIKELY(myContext->state == RW_UNLOCKED)) {
+         uint32 lockCount = Atomic_Read(&lock->holderCount);
+
+         MXUserDumpAndPanic(&lock->header,
+                            "%s: Non-owner release of an %s read-write lock\n",
+                            __FUNCTION__,
+                            lockCount == 0 ? "unacquired" : "acquired");
+      }
+
+
+      /*
+       * The statistics are not always atomically safe so protect them
+       * when necessary
+       */
+
+      if ((myContext->state == RW_LOCKED_FOR_READ) && lock->useNative) {
+         MXRecLockAcquire(&lock->recursiveLock, GetReturnAddress());
+      }
+
+      MXUserBasicStatsSample(&stats->heldStats, duration);
+
+      histo = Atomic_ReadPtr(&stats->heldHisto);
+
+      if (UNLIKELY(histo != NULL)) {
+         MXUserHistoSample(histo, duration);
+      }
+
+      if ((myContext->state == RW_LOCKED_FOR_READ) && lock->useNative) {
+         MXRecLockRelease(&lock->recursiveLock);
+      }
    }
 
-#if defined(MXUSER_STATS)
-   duration = holdEnd - myContext->holdStart;
-
-   /* The statistics are not atomically safe so protect them when necessary */
-   if ((myContext->state == RW_LOCKED_FOR_READ) && lock->useNative) {
-      MXRecLockAcquire(&lock->recursiveLock, GetReturnAddress());
-   }
-
-   MXUserBasicStatsSample(&lock->heldStats, duration);
-
-   histo = Atomic_ReadPtr(&lock->heldHisto);
-
-   if (UNLIKELY(histo != NULL)) {
-      MXUserHistoSample(histo, duration);
-   }
-
-   if ((myContext->state == RW_LOCKED_FOR_READ) && lock->useNative) {
-      MXRecLockRelease(&lock->recursiveLock);
-   }
-#endif
-
    MXUserReleaseTracking(&lock->header);
 
    Atomic_Dec(&lock->holderCount);
@@ -875,17 +903,15 @@ MXUser_CreateSingletonRWLock(Atomic_Ptr *lockStorage,  // IN/OUT:
    lock = (MXUserRWLock *) Atomic_ReadPtr(lockStorage);
 
    if (UNLIKELY(lock == NULL)) {
-      MXUserRWLock *before;
+      MXUserRWLock *newLock = MXUser_CreateRWLock(name, rank);
 
-      lock = MXUser_CreateRWLock(name, rank);
+      lock = (MXUserRWLock *) Atomic_ReadIfEqualWritePtr(lockStorage, NULL,
+                                                         (void *) newLock);
 
-      before = (MXUserRWLock *) Atomic_ReadIfEqualWritePtr(lockStorage, NULL,
-                                                           (void *) lock);
-
-      if (before) {
-         MXUser_DestroyRWLock(lock);
-
-         lock = before;
+      if (lock) {
+         MXUser_DestroyRWLock(newLock);
+      } else {
+         lock = (MXUserRWLock *) Atomic_ReadPtr(lockStorage);
       }
    }
 
