@@ -22,23 +22,26 @@
 #include "userlock.h"
 #include "hostinfo.h"
 #include "ulInt.h"
+#include "vm_atomic.h"
 
-#define MXUSER_REC_SIGNATURE 0x43524B4C // 'LKRC' in memory
-
-struct MXUserRecLock
+typedef struct
 {
-   MXUserHeader            header;
-   MXRecLock               recursiveLock;
-
    uint64                  holdStart;
 
-#if defined(MXUSER_STATS)
    MXUserAcquisitionStats  acquisitionStats;
    Atomic_Ptr              acquisitionHisto;
 
    MXUserBasicStats        heldStats;
    Atomic_Ptr              heldHisto;
-#endif
+} MXUserStats;
+
+#define MXUSER_REC_SIGNATURE 0x43524B4C // 'LKRC' in memory
+
+struct MXUserRecLock
+{
+   MXUserHeader         header;
+   MXRecLock            recursiveLock;
+   Atomic_Ptr           statsMem;
 
    /*
     * This is the MX recursive lock override pointer. It is used within the
@@ -51,7 +54,7 @@ struct MXUserRecLock
     *         MXUser_BindMXMutexRec was used to create the lock
     */
 
-   struct MX_MutexRec     *vmmLock;
+   struct MX_MutexRec  *vmmLock;
 };
 
 #if defined(MXUSER_STATS)
@@ -74,47 +77,51 @@ struct MXUserRecLock
 static void
 MXUserStatsActionRec(MXUserHeader *header)  // IN:
 {
-   Bool isHot;
-   Bool doLog;
-   double contentionRatio;
-
    MXUserRecLock *lock = (MXUserRecLock *) header;
+   MXUserStats *stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
 
-   /*
-    * Dump the statistics for the specified lock.
-    */
+   if (stats) {
+      Bool isHot;
+      Bool doLog;
+      double contentionRatio;
 
-   MXUserDumpAcquisitionStats(&lock->acquisitionStats, header);
+      /*
+       * Dump the statistics for the specified lock.
+       */
 
-   if (Atomic_ReadPtr(&lock->acquisitionHisto) != NULL) {
-      MXUserHistoDump(Atomic_ReadPtr(&lock->acquisitionHisto), header);
-   }
+      MXUserDumpAcquisitionStats(&stats->acquisitionStats, header);
 
-   MXUserDumpBasicStats(&lock->heldStats, header);
+      if (Atomic_ReadPtr(&stats->acquisitionHisto) != NULL) {
+         MXUserHistoDump(Atomic_ReadPtr(&stats->acquisitionHisto), header);
+      }
 
-   if (Atomic_ReadPtr(&lock->heldHisto) != NULL) {
-      MXUserHistoDump(Atomic_ReadPtr(&lock->heldHisto), header);
-   }
+      MXUserDumpBasicStats(&stats->heldStats, header);
 
-   /*
-    * Has the lock gone "hot"? If so, implement the hot actions.
-    */
+      if (Atomic_ReadPtr(&stats->heldHisto) != NULL) {
+         MXUserHistoDump(Atomic_ReadPtr(&stats->heldHisto), header);
+      }
 
-   MXUserKitchen(&lock->acquisitionStats, &contentionRatio, &isHot, &doLog);
+      /*
+       * Has the lock gone "hot"? If so, implement the hot actions.
+       */
 
-   if (isHot) {
-      MXUserForceHisto(&lock->acquisitionHisto,
-                       MXUSER_STAT_CLASS_ACQUISITION,
-                       MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
-                       MXUSER_DEFAULT_HISTO_DECADES);
-      MXUserForceHisto(&lock->heldHisto,
-                       MXUSER_STAT_CLASS_HELD,
-                       MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
-                       MXUSER_DEFAULT_HISTO_DECADES);
+      MXUserKitchen(&stats->acquisitionStats, &contentionRatio, &isHot,
+                    &doLog);
 
-      if (doLog) {
-         Log("HOT LOCK (%s); contention ratio %f\n",
-             lock->header.name, contentionRatio);
+      if (isHot) {
+         MXUserForceHisto(&stats->acquisitionHisto,
+                          MXUSER_STAT_CLASS_ACQUISITION,
+                          MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
+                          MXUSER_DEFAULT_HISTO_DECADES);
+         MXUserForceHisto(&stats->heldHisto,
+                          MXUSER_STAT_CLASS_HELD,
+                          MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
+                          MXUSER_DEFAULT_HISTO_DECADES);
+
+         if (doLog) {
+            Log("HOT LOCK (%s); contention ratio %f\n",
+                lock->header.name, contentionRatio);
+         }
       }
    }
 }
@@ -153,7 +160,6 @@ MXUserDumpRecLock(MXUserHeader *header)  // IN:
 
 #if defined(MXUSER_DEBUG)
       Warning("\tcaller 0x%p\n", lock->recursiveLock.ownerRetAddr);
-      Warning("\tVThreadID %d\n", (int) lock->recursiveLock.portableThreadID);
 #endif
    } else {
       Warning("\tvmmLock 0x%p\n", lock->vmmLock);
@@ -164,9 +170,10 @@ MXUserDumpRecLock(MXUserHeader *header)  // IN:
 /*
  *-----------------------------------------------------------------------------
  *
- * MXUser_CreateRecLock --
+ * MXUserCreateRecLock --
  *
- *      Create a recursive lock.
+ *      Create a recursive lock specifying if the lock must always be
+ *      silent.
  *
  *      Only the owner (thread) of a recursive lock may recurse on it.
  *
@@ -180,11 +187,13 @@ MXUserDumpRecLock(MXUserHeader *header)  // IN:
  *-----------------------------------------------------------------------------
  */
 
-MXUserRecLock *
-MXUser_CreateRecLock(const char *userName,  // IN:
-                     MX_Rank rank)          // IN:
+static MXUserRecLock *
+MXUserCreateRecLock(const char *userName,  // IN:
+                    MX_Rank rank,          // IN:
+                    Bool beSilent)         // IN:
 {
    char *properName;
+   MXUserStats *stats;
    MXUserRecLock *lock;
 
    lock = Util_SafeCalloc(1, sizeof(*lock));
@@ -212,16 +221,84 @@ MXUser_CreateRecLock(const char *userName,  // IN:
 #if defined(MXUSER_STATS)
    lock->header.statsFunc = MXUserStatsActionRec;
    lock->header.identifier = MXUserAllocID();
-
-   MXUserAddToList(&lock->header);
-   MXUserAcquisitionStatsSetUp(&lock->acquisitionStats);
-   MXUserBasicStatsSetUp(&lock->heldStats, MXUSER_STAT_CLASS_HELD);
 #endif
 
+   if (beSilent) {
+      stats = NULL;
+   } else {
+#if defined(MXUSER_STATS)
+      stats = Util_SafeCalloc(1, sizeof(*stats));
+
+      MXUserAcquisitionStatsSetUp(&stats->acquisitionStats);
+      MXUserBasicStatsSetUp(&stats->heldStats, MXUSER_STAT_CLASS_HELD);
+#else
+      stats = NULL;
+#endif
+   }
+
+   Atomic_WritePtr(&lock->statsMem, stats);
+
+   MXUserAddToList(&lock->header);
+
    return lock;
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MXUser_CreateRecLockSilent --
+ *
+ *      Create a recursive lock specifying if the lock must always be
+ *      silent - never logging any messages. Silent locks will never
+ *      produce any statistics, amongst the aspects of "silent".
+ *
+ *      Only the owner (thread) of a recursive lock may recurse on it.
+ *
+ * Results:
+ *      NULL  Creation failed
+ *      !NULL Creation succeeded
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+MXUserRecLock *
+MXUser_CreateRecLockSilent(const char *userName,  // IN:
+                           MX_Rank rank)          // IN:
+{
+   return MXUserCreateRecLock(userName, rank, TRUE);
+}
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MXUser_CreateRecLock --
+ *
+ *      Create a recursive lock.
+ *
+ *      Only the owner (thread) of a recursive lock may recurse on it.
+ *
+ * Results:
+ *      NULL  Creation failed
+ *      !NULL Creation succeeded
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+MXUserRecLock *
+MXUser_CreateRecLock(const char *userName,  // IN:
+                     MX_Rank rank)          // IN:
+{
+   return MXUserCreateRecLock(userName, rank, FALSE);
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -231,7 +308,7 @@ MXUser_CreateRecLock(const char *userName,  // IN:
  *
  *      When the lock is bound to a MX lock, only the MXUser "wrapper" is
  *      freed. The caller is responsible for calling MX_DestroyLockRec() on
- *      the MX lock.
+ *      the MX lock before calling this routine.
  *
  * Results:
  *      Lock is destroyed. Don't use the pointer again.
@@ -246,6 +323,8 @@ void
 MXUser_DestroyRecLock(MXUserRecLock *lock)  // IN:
 {
    if (lock != NULL) {
+      MXUserStats *stats;
+
       ASSERT(lock->header.signature == MXUSER_REC_SIGNATURE);
 
       if (lock->vmmLock == NULL) {
@@ -257,13 +336,18 @@ MXUser_DestroyRecLock(MXUserRecLock *lock)  // IN:
 
          MXRecLockDestroy(&lock->recursiveLock);
 
-#if defined(MXUSER_STATS)
          MXUserRemoveFromList(&lock->header);
-         MXUserAcquisitionStatsTearDown(&lock->acquisitionStats);
-         MXUserBasicStatsTearDown(&lock->heldStats);
-         MXUserHistoTearDown(Atomic_ReadPtr(&lock->acquisitionHisto));
-         MXUserHistoTearDown(Atomic_ReadPtr(&lock->heldHisto));
-#endif
+
+         stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
+
+         if (stats) {
+            MXUserAcquisitionStatsTearDown(&stats->acquisitionStats);
+            MXUserBasicStatsTearDown(&stats->heldStats);
+            MXUserHistoTearDown(Atomic_ReadPtr(&stats->acquisitionHisto));
+            MXUserHistoTearDown(Atomic_ReadPtr(&stats->heldHisto));
+
+            free(stats);
+         }
       }
 
       lock->header.signature = 0;  // just in case...
@@ -301,37 +385,36 @@ MXUser_AcquireRecLock(MXUserRecLock *lock)  // IN/OUT:
       ASSERT(MXUserMX_LockRec);
       (*MXUserMX_LockRec)(lock->vmmLock);
    } else {
-#if defined(MXUSER_STATS)
-      Bool contended;
-      VmTimeType begin;
-#endif
+      MXUserStats *stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
+
       /* Rank checking is only done on the first acquisition */
       MXUserAcquisitionTracking(&lock->header, TRUE);
 
-#if defined(MXUSER_STATS)
-      begin = Hostinfo_SystemTimerNS();
+      if (stats) {
+         Bool contended;
+         VmTimeType begin = Hostinfo_SystemTimerNS();
 
-      contended =
-#endif
+         contended = MXRecLockAcquire(&lock->recursiveLock,
+                                      GetReturnAddress());
 
-      MXRecLockAcquire(&lock->recursiveLock, GetReturnAddress());
+         if (MXRecLockCount(&lock->recursiveLock) == 1) {
+            MXUserHisto *histo;
+            VmTimeType value = Hostinfo_SystemTimerNS() - begin;
 
-#if defined(MXUSER_STATS)
-      if (MXRecLockCount(&lock->recursiveLock) == 1) {
-         MXUserHisto *histo;
-         VmTimeType value = Hostinfo_SystemTimerNS() - begin;
+            MXUserAcquisitionSample(&stats->acquisitionStats, TRUE, contended,
+                                    value);
 
-         MXUserAcquisitionSample(&lock->acquisitionStats, contended, value);
+            histo = Atomic_ReadPtr(&stats->acquisitionHisto);
 
-         histo = Atomic_ReadPtr(&lock->acquisitionHisto);
+            if (UNLIKELY(histo != NULL)) {
+               MXUserHistoSample(histo, value);
+            }
 
-         if (UNLIKELY(histo != NULL)) {
-            MXUserHistoSample(histo, value);
+            stats->holdStart = Hostinfo_SystemTimerNS();
          }
-
-         lock->holdStart = Hostinfo_SystemTimerNS();
+      } else {
+         MXRecLockAcquire(&lock->recursiveLock, GetReturnAddress());
       }
-#endif
    }
 }
 
@@ -361,18 +444,20 @@ MXUser_ReleaseRecLock(MXUserRecLock *lock)  // IN/OUT:
       ASSERT(MXUserMX_UnlockRec);
       (*MXUserMX_UnlockRec)(lock->vmmLock);
    } else {
-#if defined(MXUSER_STATS)
-      if (MXRecLockCount(&lock->recursiveLock) == 1) {
-         VmTimeType value = Hostinfo_SystemTimerNS() - lock->holdStart;
-         MXUserHisto *histo = Atomic_ReadPtr(&lock->heldHisto);
+      MXUserStats *stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
 
-         MXUserBasicStatsSample(&lock->heldStats, value);
+      if (stats) {
+         if (MXRecLockCount(&lock->recursiveLock) == 1) {
+            VmTimeType value = Hostinfo_SystemTimerNS() - stats->holdStart;
+            MXUserHisto *histo = Atomic_ReadPtr(&stats->heldHisto);
 
-         if (UNLIKELY(histo != NULL)) {
-            MXUserHistoSample(histo, value);
+            MXUserBasicStatsSample(&stats->heldStats, value);
+
+            if (UNLIKELY(histo != NULL)) {
+               MXUserHistoSample(histo, value);
+            }
          }
       }
-#endif
 
       if (!MXRecLockIsOwner(&lock->recursiveLock)) {
          uint32 lockCount = MXRecLockCount(&lock->recursiveLock);
@@ -424,30 +509,24 @@ MXUser_TryAcquireRecLock(MXUserRecLock *lock)  // IN/OUT:
       ASSERT(MXUserMX_TryLockRec);
       success = (*MXUserMX_TryLockRec)(lock->vmmLock);
    } else {
-#if defined(MXUSER_STATS)
-      VmTimeType begin;
-#endif
+      MXUserStats *stats;
 
       if (MXUserTryAcquireFail(lock->header.name)) {
          return FALSE;
       }
 
-#if defined(MXUSER_STATS)
-      begin = Hostinfo_SystemTimerNS();
-#endif
-
       success = MXRecLockTryAcquire(&lock->recursiveLock, GetReturnAddress());
 
       if (success) {
-#if defined(MXUSER_STATS)
-         if (MXRecLockCount(&lock->recursiveLock) == 1) {
-            MXUserAcquisitionSample(&lock->acquisitionStats, FALSE,
-                                  Hostinfo_SystemTimerNS() - begin);
-         }
-#endif
-
          MXUserAcquisitionTracking(&lock->header, FALSE);
       }
+
+      stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
+
+      if (stats) {
+         MXUserAcquisitionSample(&stats->acquisitionStats, success,
+                                 !success, 0ULL);
+      }
    }
 
    return success;
@@ -513,44 +592,52 @@ MXUser_ControlRecLock(MXUserRecLock *lock,  // IN/OUT:
    Bool result;
 
    ASSERT(lock && (lock->header.signature == MXUSER_REC_SIGNATURE));
-   ASSERT(lock->vmmLock == NULL);  // only unbound locks
 
    switch (command) {
-#if defined(MXUSER_STATS)
    case MXUSER_CONTROL_ACQUISITION_HISTO: {
-      va_list a;
-      uint64 minValue;
-      uint32 decades;
+      MXUserStats *stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
 
-      va_start(a, command);
-      minValue = va_arg(a, uint64);
-      decades = va_arg(a, uint32);
-      va_end(a);
+      if (stats && (lock->vmmLock == NULL)) {
+         va_list a;
+         uint64 minValue;
+         uint32 decades;
+         va_start(a, command);
+         minValue = va_arg(a, uint64);
+         decades = va_arg(a, uint32);
+         va_end(a);
 
-      MXUserForceHisto(&lock->acquisitionHisto, MXUSER_STAT_CLASS_ACQUISITION,
-                       minValue, decades);
+         MXUserForceHisto(&stats->acquisitionHisto,
+                          MXUSER_STAT_CLASS_ACQUISITION, minValue, decades);
 
-      result = TRUE;
+         result = TRUE;
+      } else {
+         result = FALSE;
+      }
       break;
    }
 
    case MXUSER_CONTROL_HELD_HISTO: {
-      va_list a;
-      uint64 minValue;
-      uint32 decades;
+      MXUserStats *stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
 
-      va_start(a, command);
-      minValue = va_arg(a, uint64);
-      decades = va_arg(a, uint32);
-      va_end(a);
+      if (stats && (lock->vmmLock == NULL)) {
+         va_list a;
+         uint64 minValue;
+         uint32 decades;
+         va_start(a, command);
+         minValue = va_arg(a, uint64);
+         decades = va_arg(a, uint32);
+         va_end(a);
 
-      MXUserForceHisto(&lock->heldHisto, MXUSER_STAT_CLASS_HELD,
-                       minValue, decades);
+         MXUserForceHisto(&stats->heldHisto, MXUSER_STAT_CLASS_HELD,
+                          minValue, decades);
       
-      result = TRUE;
+         result = TRUE;
+      } else {
+         result = FALSE;
+      }
+
       break; 
    }
-#endif
 
    default:
       result = FALSE;
@@ -591,17 +678,15 @@ MXUser_CreateSingletonRecLock(Atomic_Ptr *lockStorage,  // IN/OUT:
    lock = (MXUserRecLock *) Atomic_ReadPtr(lockStorage);
 
    if (UNLIKELY(lock == NULL)) {
-      MXUserRecLock *before;
+      MXUserRecLock *newLock = MXUser_CreateRecLock(name, rank);
 
-      lock = MXUser_CreateRecLock(name, rank);
+      lock = (MXUserRecLock *) Atomic_ReadIfEqualWritePtr(lockStorage, NULL,
+                                                          (void *) newLock);
 
-      before = (MXUserRecLock *) Atomic_ReadIfEqualWritePtr(lockStorage, NULL,
-                                                            (void *) lock);
-
-      if (before) {
-         MXUser_DestroyRecLock(lock);
-
-         lock = before;
+      if (lock) {
+         MXUser_DestroyRecLock(newLock);
+      } else {
+         lock = (MXUserRecLock *) Atomic_ReadPtr(lockStorage);
       }
    }
 
@@ -805,6 +890,8 @@ MXUser_BindMXMutexRec(struct MX_MutexRec *mutex,  // IN:
    lock->header.identifier = MXUserAllocID();
 #endif
 
+   Atomic_WritePtr(&lock->statsMem, NULL);
+
    lock->vmmLock = mutex;
 
    return lock;
@@ -813,7 +900,7 @@ MXUser_BindMXMutexRec(struct MX_MutexRec *mutex,  // IN:
 
 #if defined(VMX86_VMX)
 #include "mutex.h"
-#include "mutexRank.h"
+#include "mutexRankVMX.h"
 
 /*
  *----------------------------------------------------------------------------
