@@ -39,12 +39,10 @@
 #include "util.h"
 #include "userlock.h"
 #include "ulInt.h"
+#include "hostinfo.h"
 #if defined(_WIN32)
 #include "win32u.h"
 #endif
-#if defined(__APPLE__)
-#include "hostinfo.h"
-#endif
 
 #define MXUSER_SEMA_SIGNATURE 0x414D4553 // 'SEMA' in memory
 #define MXUSER_A_BILLION (1000 * 1000 * 1000)
@@ -59,12 +57,18 @@ typedef sem_t NativeSemaphore;
 #endif
 #endif
 
+typedef struct
+{
+   MXUserAcquisitionStats  acquisitionStats;
+   Atomic_Ptr              acquisitionHisto;
+} MXUserStats;
+
 struct MXUserSemaphore
 {
    MXUserHeader     header;
    Atomic_uint32    activeUserCount;
-
    NativeSemaphore  nativeSemaphore;
+   Atomic_Ptr       statsMem;
 };
 
 
@@ -380,6 +384,67 @@ MXUserUp(NativeSemaphore *sema)  // IN:
 #endif  // _WIN32
 
 
+#if defined(MXUSER_STATS)
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MXUserStatsActionSema --
+ *
+ *      Perform the statistics action for the specified semaphore.
+ *
+ * Results:
+ *      As above.
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static void
+MXUserStatsActionSema(MXUserHeader *header)  // IN:
+{
+   MXUserSemaphore *sema = (MXUserSemaphore *) header;
+   MXUserStats *stats = (MXUserStats *) Atomic_ReadPtr(&sema->statsMem);
+
+   if (stats) {
+      Bool isHot;
+      Bool doLog;
+      double contentionRatio;
+
+      /*
+       * Dump the statistics for the specified semaphore.
+       */
+
+      MXUserDumpAcquisitionStats(&stats->acquisitionStats, header);
+
+      if (Atomic_ReadPtr(&stats->acquisitionHisto) != NULL) {
+         MXUserHistoDump(Atomic_ReadPtr(&stats->acquisitionHisto), header);
+      }
+
+      /*
+       * Has the semaphore gone "hot"? If so, implement the hot actions.
+       */
+
+      MXUserKitchen(&stats->acquisitionStats, &contentionRatio, &isHot,
+                    &doLog);
+
+      if (isHot) {
+         MXUserForceHisto(&stats->acquisitionHisto,
+                          MXUSER_STAT_CLASS_ACQUISITION,
+                          MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
+                          MXUSER_DEFAULT_HISTO_DECADES);
+
+         if (doLog) {
+            Log("HOT SEMAPHORE (%s); contention ratio %f\n",
+                sema->header.name, contentionRatio);
+         }
+      }
+   }
+}
+#endif
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -447,18 +512,27 @@ MXUser_CreateSemaphore(const char *userName,  // IN:
    }
 
    if (LIKELY(MXUserInit(&sema->nativeSemaphore) == 0)) {
+      MXUserStats *stats;
+
       sema->header.name = properName;
       sema->header.signature = MXUSER_SEMA_SIGNATURE;
       sema->header.rank = rank;
       sema->header.dumpFunc = MXUserDumpSemaphore;
 
 #if defined(MXUSER_STATS)
-      sema->header.statsFunc = NULL;
+      sema->header.statsFunc = MXUserStatsActionSema;
       sema->header.identifier = MXUserAllocID();
 
-      MXUserAddToList(&sema->header);
+      stats = Util_SafeCalloc(1, sizeof(*stats));
+
+      MXUserAcquisitionStatsSetUp(&stats->acquisitionStats);
+#else
+      stats = NULL;
 #endif
 
+      Atomic_WritePtr(&sema->statsMem, stats);
+
+      MXUserAddToList(&sema->header);
    } else {
       free(properName);
       free(sema);
@@ -490,6 +564,7 @@ MXUser_DestroySemaphore(MXUserSemaphore *sema)  // IN:
 {
    if (LIKELY(sema != NULL)) {
       int err;
+      MXUserStats *stats;
 
       ASSERT(sema->header.signature == MXUSER_SEMA_SIGNATURE);
 
@@ -506,9 +581,16 @@ MXUser_DestroySemaphore(MXUserSemaphore *sema)  // IN:
                             __FUNCTION__, err);
       }
 
-#if defined(MXUSER_STATS)
       MXUserRemoveFromList(&sema->header);
-#endif
+
+      stats = (MXUserStats *) Atomic_ReadPtr(&sema->statsMem);
+
+      if (stats) {
+         MXUserAcquisitionStatsTearDown(&stats->acquisitionStats);
+         MXUserHistoTearDown(Atomic_ReadPtr(&stats->acquisitionHisto));
+
+         free(stats);
+      }
 
       sema->header.signature = 0;  // just in case...
       free((void *) sema->header.name);  // avoid const warnings
@@ -540,6 +622,7 @@ void
 MXUser_DownSemaphore(MXUserSemaphore *sema)  // IN/OUT:
 {
    int err;
+   MXUserStats *stats;
 
    ASSERT(sema && (sema->header.signature == MXUSER_SEMA_SIGNATURE));
 
@@ -547,7 +630,36 @@ MXUser_DownSemaphore(MXUserSemaphore *sema)  // IN/OUT:
 
    MXUserAcquisitionTracking(&sema->header, TRUE);  // rank checking
 
-   err = MXUserDown(&sema->nativeSemaphore);
+   stats = (MXUserStats *) Atomic_ReadPtr(&sema->statsMem);
+
+   if (stats) { 
+      Bool tryDownSuccess = FALSE;
+      VmTimeType begin = Hostinfo_SystemTimerNS();
+
+      err = MXUserTryDown(&sema->nativeSemaphore, &tryDownSuccess);
+
+      if (LIKELY(err == 0)) {
+         if (!tryDownSuccess) {
+            err = MXUserDown(&sema->nativeSemaphore);
+         }
+
+         if (LIKELY(err == 0)) {
+            MXUserHisto *histo;
+            VmTimeType value = Hostinfo_SystemTimerNS() - begin;
+
+            MXUserAcquisitionSample(&stats->acquisitionStats, TRUE,
+                                    !tryDownSuccess, value);
+
+            histo = Atomic_ReadPtr(&stats->acquisitionHisto);
+
+            if (UNLIKELY(histo != NULL)) {
+               MXUserHistoSample(histo, value);
+            }
+         }
+      }
+   } else {
+      err = MXUserDown(&sema->nativeSemaphore);
+   }
 
    if (UNLIKELY(err != 0)) {
       MXUserDumpAndPanic(&sema->header, "%s: Internal error (%d)\n",
@@ -584,6 +696,7 @@ MXUser_TimedDownSemaphore(MXUserSemaphore *sema,  // IN/OUT:
                           uint32 msecWait)        // IN:
 {
    int err;
+   MXUserStats *stats;
    Bool downOccurred = FALSE;
 
    ASSERT(sema && (sema->header.signature == MXUSER_SEMA_SIGNATURE));
@@ -592,7 +705,40 @@ MXUser_TimedDownSemaphore(MXUserSemaphore *sema,  // IN/OUT:
 
    MXUserAcquisitionTracking(&sema->header, TRUE);  // rank checking
 
-   err = MXUserTimedDown(&sema->nativeSemaphore, msecWait, &downOccurred);
+   stats = (MXUserStats *) Atomic_ReadPtr(&sema->statsMem);
+
+   if (stats) { 
+      Bool tryDownSuccess = FALSE;
+      VmTimeType begin = Hostinfo_SystemTimerNS();
+
+      err = MXUserTryDown(&sema->nativeSemaphore, &tryDownSuccess);
+
+      if (LIKELY(err == 0)) {
+         if (tryDownSuccess) {
+            downOccurred = TRUE;
+         } else {
+            err = MXUserTimedDown(&sema->nativeSemaphore, msecWait,
+                                  &downOccurred);
+         }
+
+         if (LIKELY(err == 0)) {
+            VmTimeType value = Hostinfo_SystemTimerNS() - begin;
+
+            MXUserAcquisitionSample(&stats->acquisitionStats, downOccurred,
+                                    !tryDownSuccess, value);
+
+            if (downOccurred) {
+               MXUserHisto *histo = Atomic_ReadPtr(&stats->acquisitionHisto);
+
+               if (UNLIKELY(histo != NULL)) {
+                  MXUserHistoSample(histo, value);
+               }
+            }
+         }
+      }
+   } else {
+      err = MXUserTimedDown(&sema->nativeSemaphore, msecWait, &downOccurred);
+   }
 
    if (UNLIKELY(err != 0)) {
       MXUserDumpAndPanic(&sema->header, "%s: Internal error (%d)\n",
@@ -633,6 +779,7 @@ Bool
 MXUser_TryDownSemaphore(MXUserSemaphore *sema)  // IN/OUT:
 {
    int err;
+   MXUserStats *stats;
    Bool downOccurred = FALSE;
 
    ASSERT(sema && (sema->header.signature == MXUSER_SEMA_SIGNATURE));
@@ -646,6 +793,13 @@ MXUser_TryDownSemaphore(MXUserSemaphore *sema)  // IN/OUT:
                          __FUNCTION__, err);
    }
 
+   stats = (MXUserStats *) Atomic_ReadPtr(&sema->statsMem);
+
+   if (stats) {
+      MXUserAcquisitionSample(&stats->acquisitionStats, downOccurred,
+                              !downOccurred, 0ULL);
+   }
+
    Atomic_Dec(&sema->activeUserCount);
 
    return downOccurred;
@@ -658,7 +812,7 @@ MXUser_TryDownSemaphore(MXUserSemaphore *sema)  // IN/OUT:
  * MXUser_UpSemaphore --
  *
  *      Perform an up (V; verhogen; "increase") operation on a semaphore.
- *
+ * 
  * Results:
  *      The semaphore count is incremented. Any thread waiting on the
  *      semaphore is awoken.
@@ -721,18 +875,16 @@ MXUser_CreateSingletonSemaphore(Atomic_Ptr *semaStorage,  // IN/OUT:
    sema = (MXUserSemaphore *) Atomic_ReadPtr(semaStorage);
 
    if (UNLIKELY(sema == NULL)) {
-      MXUserSemaphore *before;
+      MXUserSemaphore *newSema = MXUser_CreateSemaphore(name, rank);
 
-      sema = MXUser_CreateSemaphore(name, rank);
+      sema = (MXUserSemaphore *) Atomic_ReadIfEqualWritePtr(semaStorage,
+                                                            NULL,
+                                                            (void *) newSema);
 
-      before = (MXUserSemaphore *) Atomic_ReadIfEqualWritePtr(semaStorage,
-                                                              NULL,
-                                                              (void *) sema);
-
-      if (before) {
-         MXUser_DestroySemaphore(sema);
-
-         sema = before;
+      if (sema) {
+         MXUser_DestroySemaphore(newSema);
+      } else {
+         sema = (MXUserSemaphore *) Atomic_ReadPtr(semaStorage);
       }
    }
 
