@@ -30,41 +30,31 @@
 #include "hostinfo.h"
 #include "log.h"
 #include "logFixed.h"
+#if defined(MXUSER_STATS)
+#include "statsLog.h"
+#endif
 
 #define BINS_PER_DECADE 100
 
 static double mxUserContentionRatio = 0.0;  // always "off"
 static uint64 mxUserContentionCount = 0;    // always "off"
 
+#if defined(MXUSER_STATS)
 static Atomic_Ptr mxLockMemPtr;   // internal singleton lock
 static ListItem *mxUserLockList;  // list of all MXUser locks
-
-typedef struct {
-   void   *address;
-   uint64  timeValue;
-} TopOwner;
-
-#define TOPOWNERS 10
+#endif
 
 struct MXUserHisto {
-   char     *typeName;               // Type (name) of histogram
-   uint64   *binData;                // Hash table bins
-   uint64    totalSamples;           // Population sample size
-   uint64    minValue;               // Min value allowed
-   uint64    maxValue;               // Max value allowed
-   uint32    numBins;                // Number of histogram bins
-   TopOwner  ownerArray[TOPOWNERS];  // List of top owners
+   char    *typeName;      // Type (name) of histogram
+   uint64  *binData;       // hash table bins
+   uint64   totalSamples;  // Population sample size
+   uint64   minValue;      // min value allowed
+   uint64   maxValue;      // max value allowed
+   uint32   numBins;       // number of histogram bins
 };
 
-static Bool    mxUserTrackHeldTimes = FALSE;
-static char   *mxUserHistoLine = NULL;
-static uint32  mxUserMaxLineLength = 0;
-static void   *mxUserStatsContext = NULL;
-static void  (*mxUserStatsFunc)(void *context,
-                               const char *fmt,
-                               va_list ap) = NULL;
-
 
+#if defined(MXUSER_STATS)
 /*
  *-----------------------------------------------------------------------------
  *
@@ -88,8 +78,7 @@ MXUserAddToList(MXUserHeader *header)  // IN:
 
    /* Tolerate a failure. This is too low down to log */
    if (listLock) {
-      MXRecLockAcquire(listLock,
-                       NULL);  // non-stats
+      MXRecLockAcquire(listLock, GetReturnAddress());
       LIST_QUEUE(&header->item, &mxUserLockList);
       MXRecLockRelease(listLock);
    }
@@ -119,12 +108,12 @@ MXUserRemoveFromList(MXUserHeader *header)  // IN:
 
    /* Tolerate a failure. This is too low down to log */
    if (listLock) {
-      MXRecLockAcquire(listLock,
-                       NULL);  // non-stats
+      MXRecLockAcquire(listLock, GetReturnAddress());
       LIST_DEL(&header->item, &mxUserLockList);
       MXRecLockRelease(listLock);
    }
 }
+#endif
 
 
 /*
@@ -139,8 +128,7 @@ MXUserRemoveFromList(MXUserHeader *header)  // IN:
  *      (uint32) (BINS_PER_DECADE * log10(value))
  *
  * Side effects:
- *      The computed value may actually be larger than expected by a tiny
- *      amount - the log10 method is a ratio of two integers.
+ *      None
  *
  *-----------------------------------------------------------------------------
  */
@@ -156,7 +144,7 @@ MXUserHistoIndex(uint64 value)  // IN:
       uint32 numerator = 0;
       uint32 denominator = 0;
 
-      LogFixed_Base10(value, &numerator, &denominator);
+      LogFixed_Base10(value, &numerator, &denominator); 
 
       index = (BINS_PER_DECADE * numerator) / denominator;
    }
@@ -197,7 +185,7 @@ MXUserHistoSetUp(char *typeName,   // type (name) of histogram
    ASSERT(decades > 0);
    ASSERT((minValue != 0) && ((minValue == 1) || ((minValue % 10) == 0)));
 
-   histo = Util_SafeCalloc(sizeof(*histo), 1);
+   histo = Util_SafeMalloc(sizeof(*histo));
 
    histo->typeName = Util_SafeStrdup(typeName);
    histo->numBins = BINS_PER_DECADE * decades;
@@ -263,10 +251,8 @@ MXUserHistoTearDown(MXUserHisto *histo)  // IN:
 
 void
 MXUserHistoSample(MXUserHisto *histo,  // IN/OUT:
-                  uint64 durationNS,   // IN:
-                  void *ownerRetAddr)  // IN:
+                  uint64 durationNS)   // IN:
 {
-   uint32 i;
    uint32 index;
 
    ASSERT(histo);
@@ -276,65 +262,16 @@ MXUserHistoSample(MXUserHisto *histo,  // IN/OUT:
    if (durationNS < histo->minValue) {
       index = 0;
    } else {
-      index = MXUserHistoIndex(durationNS / histo->minValue);
-
-      if (index > histo->numBins - 1) {
+      if (durationNS > histo->maxValue) {
          index = histo->numBins - 1;
+      } else {
+         index = MXUserHistoIndex(durationNS / histo->minValue);
       }
    }
 
    ASSERT(index < histo->numBins);
 
    histo->binData[index]++;
-
-   index = 0;
-
-   for (i = 0; i < TOPOWNERS; i++) {
-      if (histo->ownerArray[i].address == ownerRetAddr) {
-         index = i;
-         break;
-      }
-
-      if (histo->ownerArray[i].timeValue <
-          histo->ownerArray[index].timeValue) {
-         index = i;
-      }
-   }
-
-   if (durationNS > histo->ownerArray[index].timeValue) {
-      histo->ownerArray[index].address = ownerRetAddr;
-      histo->ownerArray[index].timeValue = durationNS;
-   }
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * MXUserStatsLog --
- *
- *      Output the statistics data
- *
- * Results:
- *      As above
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-static INLINE void
-MXUserStatsLog(const char *fmt,  // IN:
-               ...)              // IN:
-{
-   va_list ap;
-
-   ASSERT(mxUserStatsFunc);
-
-   va_start(ap, fmt);
-   (*mxUserStatsFunc)(mxUserStatsContext, fmt, ap);
-   va_end(ap);
 }
 
 
@@ -361,16 +298,30 @@ MXUserHistoDump(MXUserHisto *histo,    // IN:
    ASSERT(header);
    ASSERT(histo);
 
+#if defined(MXUSER_STATS)
    if (histo->totalSamples) {
       char *p;
       uint32 i;
       uint32 spaceLeft;
 
-      ASSERT(mxUserHistoLine);
+      static uint32 maxLine = 0;
+      static char *histoLine = NULL;
 
-      i = Str_Sprintf(mxUserHistoLine, mxUserMaxLineLength,
+      /*
+       * Statistics are reported from a single thread. This avoids allocating
+       * a potentially large buffer on the stack.
+       */ 
+
+      if (maxLine == 0) {
+         maxLine = Log_MaxLineLength();  // includes terminating NUL
+         ASSERT(maxLine >= 1024);        // assert a rational minimum
+
+         histoLine = Util_SafeMalloc(maxLine);
+      }
+
+      i = Str_Sprintf(histoLine, maxLine,
                       "MXUser: h l=%u t=%s min=%"FMT64"u max=%"FMT64"u\n",
-                      header->serialNumber, histo->typeName, histo->minValue,
+                      header->identifier, histo->typeName, histo->minValue,
                       histo->maxValue);
 
       /*
@@ -379,8 +330,8 @@ MXUserHistoDump(MXUserHisto *histo,    // IN:
        * properly terminated no matter what happens.
        */
 
-      p = &mxUserHistoLine[i - 1];
-      spaceLeft = mxUserMaxLineLength - i - 2;
+      p = &histoLine[i - 1];
+      spaceLeft = maxLine - i - 2;
 
       /* Add as many histogram bins as possible within the line limitations */
       for (i = 0; i < histo->numBins; i++) {
@@ -388,8 +339,10 @@ MXUserHistoDump(MXUserHisto *histo,    // IN:
             uint32 len;
             char binEntry[32];
 
-            len = Str_Sprintf(binEntry, sizeof binEntry, " %u-%"FMT64"u\n",
-                              i, histo->binData[i]);
+            Str_Sprintf(binEntry, sizeof binEntry, " %u-%"FMT64"u\n",
+                        i, histo->binData[i]);
+
+            len = strlen(binEntry);
 
             if (len < spaceLeft) {
                /*
@@ -409,45 +362,9 @@ MXUserHistoDump(MXUserHisto *histo,    // IN:
          }
       }
 
-      MXUserStatsLog("%s", mxUserHistoLine);
-
-      i = Str_Sprintf(mxUserHistoLine, mxUserMaxLineLength,
-                      "MXUser: ht l=%u t=%s\n", header->serialNumber,
-                      histo->typeName);
-
-      p = &mxUserHistoLine[i - 1];
-      spaceLeft = mxUserMaxLineLength - i - 2;
-
-      for (i = 0; i < TOPOWNERS; i++) {
-         if (histo->ownerArray[i].address != NULL) {
-            uint32 len;
-            char binEntry[32];
-
-            /* Use a debugger to change the address to a symbol */
-            len = Str_Sprintf(binEntry, sizeof binEntry, " %p-%"FMT64"u\n",
-                              histo->ownerArray[i].address,
-                              histo->ownerArray[i].timeValue);
-
-            if (len < spaceLeft) {
-               /*
-                * Append the address, time value pair to the end of the
-                * string. This includes the terminating "\n\0". Update the
-                * pointer to the next free place to point to the '\n'. If
-                * another entry is made, things work out properly. If not
-                * the string is properly terminated as a line.
-                */
-
-               Str_Strcpy(p, binEntry, len + 1);
-               p += len - 1;
-               spaceLeft -= len;
-            } else {
-               break;
-            }
-         }
-      }
-
-      MXUserStatsLog("%s", mxUserHistoLine);
+      StatsLog(histoLine);
    }
+#endif
 }
 
 
@@ -535,28 +452,11 @@ MXUserBasicStatsSetUp(MXUserBasicStats *stats,  // IN/OUT:
  *-----------------------------------------------------------------------------
  */
 
-static double
-MXUserSqrt(double x)  // IN: hack until next round when FP goes away
-{
-   double xn;
-   double xn1 = x;
-
-   if (x == 0.0) {
-      return 0.0;
-   }
-
-   do {
-      xn = xn1;
-      xn1 = (xn + x/xn) / 2.0;
-   } while (fabs(xn1 - xn) > 1E-10);
-
-   return xn1;
-}
-
 void
 MXUserDumpBasicStats(MXUserBasicStats *stats,  // IN:
                      MXUserHeader *header)     // IN:
 {
+#if defined(MXUSER_STATS)
    uint64 stdDev;
 
    if (stats->numSamples < 2) {
@@ -581,14 +481,15 @@ MXUserDumpBasicStats(MXUserBasicStats *stats,  // IN:
       mean = ((double) stats->timeSum) / num;
       variance = (stats->timeSquaredSum - (num*mean*mean)) / (num - 1.0);
 
-      stdDev = (variance < 0.0) ? 0 : (uint64) (MXUserSqrt(variance) + 0.5);
+      stdDev = (variance < 0.0) ? 0 : (uint64) (sqrt(variance) + 0.5);
    }
 
-   MXUserStatsLog("MXUser: e l=%u t=%s c=%"FMT64"u min=%"FMT64"u "
-                  "max=%"FMT64"u mean=%"FMT64"u sd=%"FMT64"u\n",
-                  header->serialNumber, stats->typeName,
-                  stats->numSamples, stats->minTime, stats->maxTime,
-                  stats->timeSum/stats->numSamples, stdDev);
+   StatsLog("MXUser: e l=%u t=%s c=%"FMT64"u min=%"FMT64"u "
+            "max=%"FMT64"u mean=%"FMT64"u sd=%"FMT64"u\n",
+            header->identifier, stats->typeName,
+            stats->numSamples, stats->minTime, stats->maxTime,
+            stats->timeSum/stats->numSamples, stdDev);
+#endif
 }
 
 
@@ -709,20 +610,22 @@ void
 MXUserDumpAcquisitionStats(MXUserAcquisitionStats *stats,  // IN:
                            MXUserHeader *header)           // IN:
 {
+#if defined(MXUSER_STATS)
    if (stats->numAttempts > 0) {
       if (stats->numSuccesses > 0) {
          MXUserDumpBasicStats(&stats->basicStats, header);
       }
 
-      MXUserStatsLog("MXUser: ce l=%u a=%"FMT64"u s=%"FMT64"u sc=%"FMT64"u "
-                     "sct=%"FMT64"u t=%"FMT64"u\n",
-                     header->serialNumber,
-                     stats->numAttempts,
-                     stats->numSuccesses,
-                     stats->numSuccessesContended,
-                     stats->successContentionTime,
-                     stats->totalContentionTime);
+      StatsLog("MXUser: ce l=%u a=%"FMT64"u s=%"FMT64"u sc=%"FMT64"u "
+               "sct=%"FMT64"u t=%"FMT64"u\n",
+               header->identifier,
+               stats->numAttempts,
+               stats->numSuccesses,
+               stats->numSuccessesContended,
+               stats->successContentionTime,
+               stats->totalContentionTime);
    }
+#endif
 }
 
 
@@ -905,75 +808,8 @@ MXUserForceHisto(Atomic_Ptr *histoPtr,  // IN/OUT:
    }
 }
 
-/*
- *-----------------------------------------------------------------------------
- *
- * MXUserStatsMode --
- *
- *      What's to be done with statistics?
- *
- * Results:
- *      0  Statstics are disabled
- *      1  Collect statistics without tracking held times
- *      2  Collect statistics with track held times
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-uint32
-MXUserStatsMode(void)
-{
-   if (vmx86_stats && (mxUserStatsFunc != NULL) && (mxUserMaxLineLength > 0)) {
-      return mxUserTrackHeldTimes ? 2 : 1;
-   } else {
-      return 0;
-   }
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * MXUser_SetStatsFunc --
- *
- *      Establish statistics taking and reporting. This is done by registering
- *      a statistics context, a reporting function and a maximum line length.
- *
- *      A maxLineLength of zero (0) and/or a statsFunc of NULL will
- *      disable/prevent statistics gathering.
- *
- * Results:
- *      As above
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-void
-MXUser_SetStatsFunc(void *context,                    // IN:
-                    uint32 maxLineLength,             // IN:
-                    Bool trackHeldTimes,              // IN:
-                    void (*statsFunc)(void *context,  // IN:
-                                      const char *fmt,
-                                      va_list ap))
-{
-   ASSERT(maxLineLength >= 1024);   // assert a rational minimum
-
-   free(mxUserHistoLine);
-   mxUserHistoLine = Util_SafeMalloc(maxLineLength);
-
-   mxUserStatsContext = context;
-   mxUserMaxLineLength = maxLineLength;
-   mxUserStatsFunc = statsFunc;
-   mxUserTrackHeldTimes = trackHeldTimes;
-}
-
 
+#if defined(MXUSER_STATS)
 /*
  *-----------------------------------------------------------------------------
  *
@@ -1003,27 +839,23 @@ MXUser_PerLockData(void)
 {
    MXRecLock *listLock = MXUserInternalSingleton(&mxLockMemPtr);
 
-   if (mxUserStatsFunc == NULL) {
-      return;
-   }
-
-   if (listLock && MXRecLockTryAcquire(listLock)) {
+   if (listLock && MXRecLockTryAcquire(listLock, GetReturnAddress())) {
       ListItem *entry;
-      uint32 highestSerialNumber;
-      static uint32 lastReportedSerialNumber = 0;
+      uint32 highestID;
+      static uint32 lastReportedID = 0;
 
-      highestSerialNumber = lastReportedSerialNumber;
+      highestID = lastReportedID;
 
       LIST_SCAN(entry, mxUserLockList) {
          MXUserHeader *header = LIST_CONTAINER(entry, MXUserHeader, item);
 
          /* Log the ID information for a lock that did exist previously */
-         if (header->serialNumber > lastReportedSerialNumber) {
-            MXUserStatsLog("MXUser: n n=%s l=%d r=0x%x\n", header->name,
-                           header->serialNumber, header->rank);
+         if (header->identifier > lastReportedID) {
+            StatsLog("MXUser: n n=%s l=%d r=0x%x\n", header->name,
+                     header->identifier, header->rank);
 
-            if (header->serialNumber > highestSerialNumber) {
-               highestSerialNumber = header->serialNumber;
+            if (header->identifier > highestID) {
+               highestID = header->identifier;
             }
          }
 
@@ -1036,21 +868,22 @@ MXUser_PerLockData(void)
          }
       }
 
-      lastReportedSerialNumber = highestSerialNumber;
+      lastReportedID = highestID;
 
       MXRecLockRelease(listLock);
    }
 }
+#endif
 
 
 /*
  *-----------------------------------------------------------------------------
  *
- * MXUserAllocSerialNumber --
+ * MXUserAllocID --
  *
- *      Allocate and return an MXUser serial number.
+ *      Allocate and return an MXUser identifier
  *
- *      MXUser serial numbers are never recycled.
+ *      MXUser identifiers are never recycled.
  *
  * Results:
  *      As above.
@@ -1062,9 +895,9 @@ MXUser_PerLockData(void)
  */
 
 uint32
-MXUserAllocSerialNumber(void)
+MXUserAllocID(void)
 {
-   static Atomic_uint32 firstFreeSerialNumber = { 1 };  // must start not zero
+   static Atomic_uint32 firstFreeID = { 1 };  // must start not zero
 
-   return Atomic_FetchAndInc(&firstFreeSerialNumber);
+   return Atomic_FetchAndInc(&firstFreeID);
 }
