@@ -98,7 +98,7 @@ static Bool CodeSetOldIso88591ToUtf8Db(char const *bufIn, size_t sizeIn,
  *-----------------------------------------------------------------------------
  */
 
-static INLINE int
+int
 CodeSetOldGetUtf8(const char *string,  // IN: string
                   const char *end,     // IN: end of string
                   uint32 *uchar)       // OUT: the Unicode character
@@ -1001,7 +1001,7 @@ CodeSetOld_GenericToGenericDb(char const *codeIn,   // IN:
        * change bufIn so a simple cast is safe. --plangdale
        */
 
-#if defined(GLIBC_VERSION_22) || (!defined(__i386__) && !defined(__x86_64__))
+#ifdef __linux__
       status = iconv(cd, (char **)&bufIn, &sizeIn, &out, &outLeft);
 #else
       status = iconv(cd, &bufIn, &sizeIn, &out, &outLeft);
