@@ -501,9 +501,11 @@ HostinfoGetOSShortName(char *distro,         // IN: full distro name
       }
    } else if (strstr(distroLower, "suse")) {
       if (strstr(distroLower, "enterprise")) {
-         if (strstr(distroLower, "server 11")) {
+         if (strstr(distroLower, "server 11") ||
+             strstr(distroLower, "desktop 11")) {
             Str_Strcpy(distroShort, STR_OS_SLES_11, distroShortSize);
-         } else if (strstr(distroLower, "server 10")) {
+         } else if (strstr(distroLower, "server 10") ||
+                    strstr(distroLower, "desktop 10")) {
             Str_Strcpy(distroShort, STR_OS_SLES_10, distroShortSize);
          } else {
             Str_Strcpy(distroShort, STR_OS_SUSE_EN, distroShortSize);
@@ -1392,8 +1394,7 @@ HostinfoGetLoadAverage(float *avg0,  // IN/OUT:
                        float *avg1,  // IN/OUT:
                        float *avg2)  // IN/OUT:
 {
-   /* getloadavg(3) was introduced with glibc 2.2 */
-#if defined(GLIBC_VERSION_22) || defined(__APPLE__)
+#if defined(__linux__) || defined(__APPLE__)
    double avg[3];
    int res;
 
@@ -3113,7 +3114,7 @@ HostinfoSysinfo(uint64 *totalRam,  // OUT: Total RAM in bytes
  * HostinfoGetLinuxMemoryInfoInPages --
  *
  *      Obtain the minimum memory to be maintained, total memory available,
- *      and free memory available on the host (Linux or COS) in pages.
+ *      and free memory available on the host (Linux) in pages.
  *
  * Results:
  *      TRUE on success: '*minSize', '*maxSize' and '*currentSize' are set
@@ -3183,7 +3184,7 @@ HostinfoGetLinuxMemoryInfoInPages(unsigned int *minSize,      // OUT:
  *
  * HostinfoGetSwapInfoInPages --
  *
- *      Obtain the total swap and free swap on the host (Linux or COS) in
+ *      Obtain the total swap and free swap on the host (Linux) in
  *      pages.
  *
  * Results:
@@ -3311,39 +3312,6 @@ Hostinfo_GetMemoryInfoInPages(unsigned int *minSize,      // OUT:
 }
 
 
-#ifdef VMX86_SERVER
-/*
- *-----------------------------------------------------------------------------
- *
- * Hostinfo_GetCOSMemoryInfoInPages --
- *
- *      Obtain the minimum memory to be maintained, total memory available, and
- *      free memory available on the COS in pages.
- *
- * Results:
- *      TRUE on success: '*minSize', '*maxSize' and '*currentSize' are set
- *      FALSE on failure
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-Bool
-Hostinfo_GetCOSMemoryInfoInPages(unsigned int *minSize,      // OUT:
-                                 unsigned int *maxSize,      // OUT:
-                                 unsigned int *currentSize)  // OUT:
-{
-   if (HostType_OSIsPureVMK()) {
-      return FALSE;
-   } else {
-      return HostinfoGetLinuxMemoryInfoInPages(minSize, maxSize, currentSize);
-   }
-}
-#endif
-
-
 /*
  *-----------------------------------------------------------------------------
  *
