@@ -67,8 +67,6 @@ RpcInSetup(RpcChannel *chan,
 /**
  * Starts the RpcIn loop and the RpcOut channel.
  *
- * No-op if channels are already started.
- *
  * @param[in]  chan     The RPC channel instance.
  *
  * @return TRUE on success.
@@ -80,13 +78,8 @@ RpcInStart(RpcChannel *chan)
    gboolean ret = TRUE;
    BackdoorChannel *bdoor = chan->_private;
 
-   if (bdoor->outStarted) {
-      /* Already started. Make sure both channels are in sync and return. */
-      ASSERT(bdoor->in == NULL || bdoor->inStarted);
-      return ret;
-   } else {
-      ASSERT(bdoor->in == NULL || !bdoor->inStarted);
-   }
+   ASSERT(bdoor->in == NULL || !bdoor->inStarted);
+   ASSERT(!bdoor->outStarted);
 
    if (bdoor->in != NULL) {
       ret = RpcIn_start(bdoor->in, RPCIN_MAX_DELAY, RpcChannel_Error, chan);
@@ -180,7 +173,7 @@ RpcInShutdown(RpcChannel *chan)
 
 static gboolean
 RpcInSend(RpcChannel *chan,
-          char const *data,
+          char *data,
           size_t dataLen,
           char **result,
           size_t *resultLen)
