@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2010 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -565,22 +565,11 @@ MsgLoadCatalog(const char *path)
             break;
          }
 
-         /*
-          * Fix the line break to always be Unix-style, to make lib/dict
-          * happy.
-          */
-         if (line[term] == '\r') {
-            line[term] = '\n';
-            if (len > term) {
-               line[term + 1] = '\0';
-            }
-         }
-
          /*
           * If currently name is not NULL, then check if this is a continuation
           * line and, if it is, just append the contents to the current value.
           */
-         if (term > 0 && name != NULL && line[term - 1] == '"') {
+         if (name != NULL && line[term - 1] == '"') {
             for (i = 0; i < len; i++) {
                if (line[i] == '"') {
                   /* OK, looks like a continuation line. */
@@ -738,8 +727,9 @@ VMTools_BindTextDomain(const char *domain,
       dfltdir = Util_SafeStrdup(VMTOOLS_DATA_DIR);
 #else
       dfltdir = GuestApp_GetInstallPath();
+      ASSERT_MEM_ALLOC(dfltdir);
 #endif
-      catdir = (dfltdir) ? dfltdir : ".";
+      catdir = dfltdir;
    }
 
    file = g_strdup_printf("%s%smessages%s%s%s%s.vmsg",
@@ -756,7 +746,6 @@ VMTools_BindTextDomain(const char *domain,
             usrlang = Util_SafeStrdup(lang);
          }
          usrlang[sep - lang] = '\0';
-         g_free(file);
          file = g_strdup_printf("%s%smessages%s%s%s%s.vmsg",
                                 catdir, DIRSEPS, DIRSEPS, usrlang, DIRSEPS, domain);
       }
