@@ -290,7 +290,7 @@ VSockVmciHandleRead(struct sock *sk,            // IN
    PKT_FIELD(vsk, sentWaitingWrite) = FALSE;
 #endif
 
-   sk->compat_sk_write_space(sk);
+   sk->sk_write_space(sk);
 }
 
 
@@ -511,7 +511,7 @@ VSockVmciHandleWrote(struct sock *sk,            // IN
    PKT_FIELD(vsk, sentWaitingRead) = FALSE;
 #endif
 
-   sk->compat_sk_data_ready(sk, 0);
+   sk->sk_data_ready(sk, 0);
 }
 
 
@@ -615,7 +615,7 @@ VSockVmciNotifyPktPollIn(struct sock *sk,    // IN
        * We can't read right now because there is nothing in the queue.
        * Ask for notifications when there is something to read.
        */
-      if (sk->compat_sk_state == SS_CONNECTED) {
+      if (sk->sk_state == SS_CONNECTED) {
          if (!VSockVmciSendWaitingRead(sk, 1)) {
             return -1;
          }
