@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2010 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -36,18 +36,7 @@ Reload_Init(ToolsAppCtx *ctx,
             ToolsPluginData *pdata);
 
 void
-Reload_Shutdown(ToolsAppCtx *ctx,
-                ToolsPluginData *pdata);
-
-#ifndef NO_XSM
-gboolean
-SessionMgr_Init(ToolsAppCtx *ctx,
-                ToolsPluginData *pdata);
-
-void
-SessionMgr_Shutdown(ToolsAppCtx *ctx,
-                    ToolsPluginData *pdata);
-#endif
+Reload_Shutdown(ToolsAppCtx *ctx);
 
 gboolean
 X11Lock_Init(ToolsAppCtx *ctx,
@@ -58,21 +47,16 @@ XIOError_Init(ToolsAppCtx *ctx,
               ToolsPluginData *pdata);
 
 void
-XIOError_Shutdown(ToolsAppCtx *ctx,
-                  ToolsPluginData *pdata);
+XIOError_Shutdown(ToolsAppCtx *ctx);
 
 
 #if defined(DE_MAIN)
 static DesktopEventFuncs gFeatures[] = {
    { X11Lock_Init,         NULL,                      FALSE },
    { Reload_Init,          Reload_Shutdown,           FALSE },
-#ifndef NO_XSM
-   { SessionMgr_Init,      SessionMgr_Shutdown,       FALSE },
-#endif
    { XIOError_Init,        XIOError_Shutdown,         FALSE },
 };
 #endif
 
-
 #endif /* _DEFEATURES_H_ */
 
