@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2009-2016 VMware, Inc. All rights reserved.
+ * Copyright (C) 2009 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -31,19 +31,10 @@
 
 typedef struct DesktopEventFuncs {
    gboolean (*initFn)(ToolsAppCtx *, ToolsPluginData *);
-   void     (*shutdownFn)(ToolsAppCtx *ctx, ToolsPluginData *);
+   void     (*shutdownFn)(ToolsAppCtx *ctx);
    gboolean initialized;
 } DesktopEventFuncs;
 
-
-/*
- * This plugin's private data field is a GHashTable*.  Each sub-feature may use
- * that table to keep its own private state.  The following key is reserved and
- * points to the hosting application's ToolsAppCtx.
- */
-#define DE_PRIVATE_CTX  "ctx"
-
-
 /*
  * This platform-specific file defines the list of features available
  * for the current platform being built.
@@ -51,3 +42,4 @@ typedef struct DesktopEventFuncs {
 #include "deFeatures.h"
 
 #endif /* _DESKTOPEVENTSINT_H_ */
+
