@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2010 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -111,14 +111,12 @@ Reload_Init(ToolsAppCtx *ctx,
  * Unregisters the SIGUSR2 signal handler.
  *
  * @param[in] ctx   Application context.
- * @param[in] pdata Plugin data (unused).
  *
  ******************************************************************************
  */
 
 void
-Reload_Shutdown(ToolsAppCtx *ctx,
-                ToolsPluginData *pdata)
+Reload_Shutdown(ToolsAppCtx *ctx)
 {
    g_source_destroy(gReloadSrc);
    g_source_unref(gReloadSrc);
