@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010-2016 VMware, Inc. All rights reserved.
+ * Copyright (C) 2010 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -22,25 +22,17 @@
  * Handles responding to X11 I/O errors.
  */
 
-/* Include first.  Sets G_LOG_DOMAIN. */
 #include "desktopEventsInt.h"
-
-#include <sys/types.h>
-#include <X11/Xlib.h>
-
-#include <glib.h>
-#include <glib-object.h>
 #include <stdlib.h>
+#include <sys/types.h>
 #include <unistd.h>
-
-#include "vmware/tools/desktopevents.h"
+#include <X11/Xlib.h>
 
 
 static int gParentPid;
 static ToolsAppCtx *gCtx;
 static XIOErrorHandler gOrigHandler;
 
-
 /*
  ******************************************************************************
  * DEXIOErrorHandler --                                                 */ /**
@@ -68,13 +60,6 @@ DEXIOErrorHandler(Display *dpy)
    if (my_pid == gParentPid) {
       g_debug("%s", __func__);
 
-      /*
-       * Inform clients capable of/interested in quick'n'dirty cleanup upon an
-       * X I/O error.
-       */
-      g_message("Emitting %s due to X I/O error.\n", TOOLS_CORE_SIG_XIOERROR);
-      g_signal_emit_by_name(gCtx->serviceObj, TOOLS_CORE_SIG_XIOERROR, gCtx);
-
       /*
        * XXX: the really correct thing to do here would be to properly stop all
        * plugins so that capabilities are unset and all other "clean shutdown"
@@ -96,6 +81,7 @@ DEXIOErrorHandler(Display *dpy)
       if (gCtx->rpc != NULL) {
          RpcChannel_Stop(gCtx->rpc);
       }
+      Reload_Do();
       exit(EXIT_FAILURE);
    } else {
       /*
@@ -131,18 +117,6 @@ XIOError_Init(ToolsAppCtx *ctx,
    gCtx = ctx;
    gParentPid = getpid();
    gOrigHandler = XSetIOErrorHandler(DEXIOErrorHandler);
-
-   g_signal_new(TOOLS_CORE_SIG_XIOERROR,
-                G_OBJECT_TYPE(ctx->serviceObj),
-                0,      // GSignalFlags
-                0,      // class offset
-                NULL,   // accumulator
-                NULL,   // accu_data
-                g_cclosure_marshal_VOID__POINTER,
-                G_TYPE_NONE,
-                1,
-                G_TYPE_POINTER);
-
    return TRUE;
 }
 
@@ -154,14 +128,12 @@ XIOError_Init(ToolsAppCtx *ctx,
  * Shutdown function, restores the original X I/O error handler.
  *
  * @param[in] ctx   Application context.
- * @param[in] pdata Plugin data (unused).
  *
  ******************************************************************************
  */
 
 void
-XIOError_Shutdown(ToolsAppCtx *ctx,
-                  ToolsPluginData *pdata)
+XIOError_Shutdown(ToolsAppCtx *ctx)
 {
    XSetIOErrorHandler(gOrigHandler);
    gCtx = NULL;
