@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2010 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -35,7 +35,6 @@ class CopyPasteDnDWrapper
 public:
    ~CopyPasteDnDWrapper();
    static CopyPasteDnDWrapper *GetInstance();
-   static void Destroy();
    gboolean RegisterCP();
    void UnregisterCP();
    gboolean RegisterDnD();
@@ -59,7 +58,6 @@ public:
    void Init(ToolsAppCtx *ctx);
    void PointerInit(void);
    ToolsAppCtx *GetToolsAppCtx() {return m_ctx;};
-   uint32 GetCaps();
 private:
    /*
     * We're a singleton, so it is a compile time error to call these.
