@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2010 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -54,7 +54,6 @@ public:
    virtual void UnregisterDnD();
    virtual void DnDVersionChanged(int version);
    virtual void CopyPasteVersionChanged(int version);
-   virtual uint32 GetCaps();
    void SetUnityMode(Bool mode) {m_dndUI->SetUnityMode(mode);};
    void SetDnDAllowed(bool allowed);
    void SetCopyPasteAllowed(bool allowed);
@@ -62,6 +61,7 @@ private:
    Gtk::Main *m_main;
    CopyPasteUIX11 *m_copyPasteUI;
    DnDUIX11 *m_dndUI;
+   DnDBlockControl m_blockCtrl;
 };
 
 #endif // __COPYPASTEDNDX11_H__
