@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2009-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2009 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -26,53 +26,13 @@
  *    http://www.jwz.org/doc/x-cut-and-paste.html
  */
 
-
-/*
- * A Word on Selection Timestamps
- *
- * ICCCM §2.6.2 Target Atoms
- *    The TIMESTAMP property is an INTEGER.
- *
- * ICCCM §2.7 Use of Selection Properties
- *    The format of INTEGER is 32.
- *
- * XGetWindowProperty(3)
- *    “If the returned format is 32, the property will be stored as an
- *    array of longs (which in a 64-bit application will be 64-bit values
- *    that are padded in the upper 4 bytes).”
- *
- * For all intents and purposes, on x86 and x86_64 X selection timestamps
- * are a 32-bit quantity. (X11/Xproto.h's xSetSelectionOwnerReq defines the
- * “time” member as the lower 32 bits of type Time.) X clients, on the
- * other hand, operate on Time as either a CARD32 (uint32) or an unsigned
- * long (i.e., on a 64-bit machine Time may occupy 8 bytes).
- *
- * Breaking it down:
- *   · When Gtk+ provides an X11 selection via Gtk::SelectionData, on a
- *     32-bit machine we'll have 4 bytes of raw data.  Everything's copacetic.
- *   · On a 64-bit machine, even if the source client provides 32 bits of
- *     timestamp data, Gtk+ will decode as an unsigned long and provide 8
- *     bytes of raw data.
- *   · On a 64-bit machine with a wacky application which actually tries
- *     to record a full 64 bits of timestamp data, Gtk+ will provide 16 bytes:
- *     <low 32 bits> <32 bits of 0s> <high 32 bits> <32 bits of 0s>.  (See
- *     PR 882322, mrxvt.)
- *
- *   In all instances, we're interested in _only_ the lowest 32 bits, so we'll
- *   ignore everything else.
- */
-
-
 #define G_LOG_DOMAIN "dndcp"
 
 #include <sys/time.h>
 #include <time.h>
-#include <cxxabi.h>
 #include "copyPasteDnDWrapper.h"
 #include "copyPasteUIX11.h"
 #include "dndFileList.hh"
-#include "guestDnDCPMgr.hh"
-#include "tracer.hh"
 
 extern "C" {
    #include "vmblock.h"
@@ -83,7 +43,9 @@ extern "C" {
    #include "cpName.h"
    #include "cpNameUtil.h"
    #include "rpcout.h"
+   #include "eventManager.h"
    #include "vmware/guestrpc/tclodefs.h"
+   #include "guestApp.h"
 }
 
 /*
@@ -125,20 +87,12 @@ CopyPasteUIX11::CopyPasteUIX11()
  : mClipboardEmpty(true),
    mHGStagingDir(""),
    mIsClipboardOwner(false),
-   mClipTime(0),
-   mPrimTime(0),
-   mLastTimestamp(0),
-   mHGGetFileStatus(DND_FILE_TRANSFER_NOT_STARTED),
+   mHGGetFilesInitiated(false),
+   mFileTransferDone(false),
    mBlockAdded(false),
    mBlockCtrl(0),
-   mInited(false),
-   mTotalFileSize(0),
-   mGetTimestampOnly(false)
+   mInited(false)
 {
-   GuestDnDCPMgr *p = GuestDnDCPMgr::GetInstance();
-   ASSERT(p);
-   mCP = p->GetCopyPasteMgr();
-   ASSERT(mCP);
 }
 
 
@@ -162,7 +116,7 @@ CopyPasteUIX11::CopyPasteUIX11()
 bool
 CopyPasteUIX11::Init()
 {
-   TRACE_CALL();
+   g_debug("%s: enter\n", __FUNCTION__);
    if (mInited) {
       return true;
    }
@@ -177,11 +131,11 @@ CopyPasteUIX11::Init()
    mListTargets.push_back(gnome);
    mListTargets.push_back(kde);
 
-   mCP->srcRecvClipChanged.connect(
+   mCP.newClipboard.connect(
       sigc::mem_fun(this, &CopyPasteUIX11::GetRemoteClipboardCB));
-   mCP->destRequestClipChanged.connect(
+   mCP.localGetClipboard.connect(
       sigc::mem_fun(this, &CopyPasteUIX11::GetLocalClipboard));
-   mCP->getFilesDoneChanged.connect(
+   mCP.localGetFilesDoneChanged.connect(
       sigc::mem_fun(this, &CopyPasteUIX11::GetLocalFilesDone));
 
    mInited = true;
@@ -207,30 +161,8 @@ CopyPasteUIX11::Init()
 
 CopyPasteUIX11::~CopyPasteUIX11()
 {
-   TRACE_CALL();
+   g_debug("%s: enter\n", __FUNCTION__);
    CPClipboard_Destroy(&mClipboard);
-   /* Any files from last unfinished file transfer should be deleted. */
-   if (DND_FILE_TRANSFER_IN_PROGRESS == mHGGetFileStatus &&
-       !mHGStagingDir.empty()) {
-      uint64 totalSize = File_GetSizeEx(mHGStagingDir.c_str());
-      if (mTotalFileSize != totalSize) {
-         g_debug("%s: deleting %s, expecting %"FMT64"d, finished %"FMT64"d\n",
-                 __FUNCTION__, mHGStagingDir.c_str(),
-                 mTotalFileSize, totalSize);
-         DnD_DeleteStagingFiles(mHGStagingDir.c_str(), FALSE);
-      } else {
-         g_debug("%s: file size match %s\n",
-                 __FUNCTION__, mHGStagingDir.c_str());
-      }
-   }
-   if (mBlockAdded) {
-      g_debug("%s: removing block for %s\n", __FUNCTION__, mHGStagingDir.c_str());
-      /* We need to make sure block subsystem has not been shut off. */
-      if (DnD_BlockIsReady(mBlockCtrl)) {
-         mBlockCtrl->RemoveBlock(mBlockCtrl->fd, mHGStagingDir.c_str());
-      }
-      mBlockAdded = false;
-   }
 }
 
 
@@ -254,9 +186,8 @@ void
 CopyPasteUIX11::VmxCopyPasteVersionChanged(RpcChannel *chan,    // IN
                                            uint32 version)      // IN
 {
-   ASSERT(mCP);
    g_debug("%s: new version is %d\n", __FUNCTION__, version);
-   mCP->VmxCopyPasteVersionChanged(version);
+   mCP.VmxCopyPasteVersionChanged(chan, version);
 }
 
 
@@ -278,21 +209,20 @@ CopyPasteUIX11::VmxCopyPasteVersionChanged(RpcChannel *chan,    // IN
  *-----------------------------------------------------------------------------
  */
 
-void
-CopyPasteUIX11::GetLocalClipboard(void)
+bool
+CopyPasteUIX11::GetLocalClipboard(CPClipboard *clip) // OUT
 {
    g_debug("%s: enter.\n", __FUNCTION__);
 
    if (mIsClipboardOwner) {
-      /* If we are clipboard owner, send a not-changed clip to host. */
-      g_debug("%s: we are owner, send unchanged clip back.\n", __FUNCTION__);
-      SendClipNotChanged();
-      return;
+      g_debug("%s: is clipboard owner, set changed to false and return.\n", __FUNCTION__);
+      CPClipboard_SetChanged(clip, FALSE);
+      return true;
    }
 
-   if (!mCP->IsCopyPasteAllowed()) {
+   if (!mCP.IsCopyPasteAllowed()) {
       g_debug("%s: copyPaste is not allowed\n", __FUNCTION__);
-      return;
+      return true;
    }
 
    Glib::RefPtr<Gtk::Clipboard> refClipboard =
@@ -301,10 +231,10 @@ CopyPasteUIX11::GetLocalClipboard(void)
    mClipTime = 0;
    mPrimTime = 0;
    mGHSelection = GDK_SELECTION_CLIPBOARD;
-   mGetTimestampOnly = false;
    g_debug("%s: retrieving timestamps\n", __FUNCTION__);
    refClipboard->request_contents(TARGET_NAME_TIMESTAMP,
       sigc::mem_fun(this, &CopyPasteUIX11::LocalClipboardTimestampCB));
+   return false;
 }
 
 
@@ -361,7 +291,11 @@ CopyPasteUIX11::LocalGetFileRequestCB(Gtk::SelectionData& sd,        // IN:
                                       guint info)                    // IN:
 {
    g_debug("%s: enter.\n", __FUNCTION__);
+   mHGCopiedUriList = "";
    VmTimeType curTime;
+   mBlockAdded = false;
+
+   sd.set(sd.get_target().c_str(), "");
 
    curTime = GetCurrentTime();
 
@@ -376,45 +310,35 @@ CopyPasteUIX11::LocalGetFileRequestCB(Gtk::SelectionData& sd,        // IN:
       return;
    }
 
-   if (!mIsClipboardOwner || !mCP->IsCopyPasteAllowed()) {
+   if (!mIsClipboardOwner || !mCP.IsCopyPasteAllowed()) {
       g_debug("%s: not clipboard ownder, or copy paste not allowed, returning.\n",
             __FUNCTION__);
-      sd.set(sd.get_target().c_str(), "");
       return;
    }
 
    g_debug("%s: Got paste request, target is %s\n", __FUNCTION__,
          sd.get_target().c_str());
 
-   if (mHGGetFileStatus != DND_FILE_TRANSFER_NOT_STARTED) {
-      /*
-       * On KDE (at least), we can see this multiple times, so ignore if
-       * we are already getting files.
-       */
-      g_debug("%s: GetFiles already started, returning uriList [%s]\n",
-              __FUNCTION__, mHGCopiedUriList.c_str());
-      sd.set(sd.get_target().c_str(), mHGCopiedUriList.c_str());
-      return;
-   } else {
+   /* Copy the files. */
+   if (!mHGGetFilesInitiated) {
       utf::string str;
       utf::string hgStagingDir;
       utf::string stagingDirName;
       utf::string pre;
       utf::string post;
       size_t index = 0;
+      mFileTransferDone = false;
 
-      hgStagingDir = static_cast<utf::string>(mCP->SrcUIRequestFiles());
+      hgStagingDir = static_cast<utf::string>(mCP.GetFiles());
       g_debug("%s: Getting files. Staging dir: %s", __FUNCTION__,
             hgStagingDir.c_str());
 
       if (0 == hgStagingDir.bytes()) {
          g_debug("%s: Can not create staging directory\n", __FUNCTION__);
-         sd.set(sd.get_target().c_str(), "");
          return;
       }
-      mHGGetFileStatus = DND_FILE_TRANSFER_IN_PROGRESS;
+      mHGGetFilesInitiated = true;
 
-      mBlockAdded = false;
       if (DnD_BlockIsReady(mBlockCtrl) && mBlockCtrl->AddBlock(mBlockCtrl->fd, hgStagingDir.c_str())) {
          g_debug("%s: add block for %s.\n",
                __FUNCTION__, hgStagingDir.c_str());
@@ -437,7 +361,6 @@ CopyPasteUIX11::LocalGetFileRequestCB(Gtk::SelectionData& sd,        // IN:
       } else {
          g_debug("%s: Unknown request target: %s\n", __FUNCTION__,
                sd.get_target().c_str());
-         sd.set(sd.get_target().c_str(), "");
          return;
       }
 
@@ -445,7 +368,6 @@ CopyPasteUIX11::LocalGetFileRequestCB(Gtk::SelectionData& sd,        // IN:
       stagingDirName = GetLastDirName(hgStagingDir);
       if (0 == stagingDirName.bytes()) {
          g_debug("%s: Can not get staging directory name\n", __FUNCTION__);
-         sd.set(sd.get_target().c_str(), "");
          return;
       }
 
@@ -468,7 +390,6 @@ CopyPasteUIX11::LocalGetFileRequestCB(Gtk::SelectionData& sd,        // IN:
 
    if (0 == mHGCopiedUriList.bytes()) {
       g_debug("%s: Can not get uri list\n", __FUNCTION__);
-      sd.set(sd.get_target().c_str(), "");
       return;
    }
 
@@ -488,9 +409,12 @@ CopyPasteUIX11::LocalGetFileRequestCB(Gtk::SelectionData& sd,        // IN:
        * apparently it only has so much patience regarding how quickly we
        * return.
        */
+      g_debug("%s no blocking driver, waiting for "
+            "HG file copy done ... mFileTransferDone is %d\n", __FUNCTION__,
+            (int) mFileTransferDone);
       CopyPasteDnDWrapper *wrapper = CopyPasteDnDWrapper::GetInstance();
       ToolsAppCtx *ctx = wrapper->GetToolsAppCtx();
-      while (mHGGetFileStatus == DND_FILE_TRANSFER_IN_PROGRESS) {
+      while (mFileTransferDone == false) {
          struct timeval tv;
 
          tv.tv_sec = 0;
@@ -499,7 +423,6 @@ CopyPasteUIX11::LocalGetFileRequestCB(Gtk::SelectionData& sd,        // IN:
          if (select(0, NULL, NULL, NULL, &tv) == -1) {
             g_debug("%s: error in select (%s).\n", __FUNCTION__,
                   strerror(errno));
-            sd.set(sd.get_target().c_str(), "");
             return;
          }
       }
@@ -536,7 +459,7 @@ CopyPasteUIX11::LocalGetTextOrRTFRequestCB(Gtk::SelectionData& sd, // IN/OUT
 {
    sd.set(sd.get_target().c_str(), "");
 
-   if (!mCP->IsCopyPasteAllowed()) {
+   if (!mCP.IsCopyPasteAllowed()) {
       return;
    }
 
@@ -547,13 +470,13 @@ CopyPasteUIX11::LocalGetTextOrRTFRequestCB(Gtk::SelectionData& sd, // IN/OUT
 
    if (target == TARGET_NAME_APPLICATION_RTF ||
        target == TARGET_NAME_TEXT_RICHTEXT) {
-      if (0 == mHGRTFData.size()) {
+      if (0 == mHGRTFData.bytes()) {
          g_debug("%s: Can not get valid RTF data\n", __FUNCTION__);
          return;
       }
 
       g_debug("%s: providing RTF data, size %"FMTSZ"u\n",
-            __FUNCTION__, mHGRTFData.size());
+            __FUNCTION__, mHGRTFData.bytes());
 
       sd.set(target.c_str(), mHGRTFData.c_str());
    }
@@ -618,20 +541,16 @@ void
 CopyPasteUIX11::LocalClipboardTimestampCB(const Gtk::SelectionData& sd)  // IN
 {
    int length = sd.get_length();
-
-   /*
-    * See “A Word on Selection Timestamps” above.
-    */
-   if (   (   sd.get_data_type().compare("INTEGER") == 0
-           || sd.get_data_type().compare("TIMESTAMP") == 0)
-       && sd.get_format() == 32
-       && length >= 4 /* sizeof uint32 */) {
-      mClipTime = reinterpret_cast<const uint32*>(sd.get_data())[0];
+   g_debug("%s: enter sd.get_length() %d.\n", __FUNCTION__,
+         length);
+   if (length == 4) {
+      mClipTime = ((uint32*) sd.get_data())[0];
+      g_debug("%s: mClipTime: %"FMT64"u.", __FUNCTION__, mClipTime);
+   } else if (length == 8) {
+      mClipTime = ((uint64*) sd.get_data())[0];
+      g_debug("%s: mClipTime: %"FMT64"u.", __FUNCTION__, mClipTime);
    } else {
-      g_debug("%s: Unable to get mClipTime (sd: len %d, type %s, fmt %d).",
-              __FUNCTION__, length,
-              length >= 0 ? sd.get_data_type().c_str() : "(n/a)",
-              sd.get_format());
+      g_debug("%s: Unable to get mClipTime.", __FUNCTION__);
    }
 
    Glib::RefPtr<Gtk::Clipboard> refClipboard
@@ -662,44 +581,21 @@ void
 CopyPasteUIX11::LocalPrimTimestampCB(const Gtk::SelectionData& sd)  // IN
 {
    int length = sd.get_length();
-
-   /*
-    * See “A Word on Selection Timestamps” above.
-    */
-   if (   (   sd.get_data_type().compare("INTEGER") == 0
-           || sd.get_data_type().compare("TIMESTAMP") == 0)
-       && sd.get_format() == 32
-       && length >= 4 /* sizeof uint32 */) {
-      mPrimTime = reinterpret_cast<const uint32*>(sd.get_data())[0];
+   g_debug("%s: enter sd.get_length() is %d.\n", __FUNCTION__, length);
+   if (length == 4) {
+      mPrimTime = ((uint32*) sd.get_data())[0];
+      g_debug("%s: mPrimTime: %"FMT64"u.", __FUNCTION__, mPrimTime);
+   } else if (length == 8) {
+      mPrimTime = ((uint64*) sd.get_data())[0];
+      g_debug("%s: mPrimTime: %"FMT64"u.", __FUNCTION__, mPrimTime);
    } else {
-      g_debug("%s: Unable to get mPrimTime (sd: len %d, type %s, fmt %d).",
-              __FUNCTION__, length,
-              length >= 0 ? sd.get_data_type().c_str() : "(n/a)",
-              sd.get_format());
-   }
-
-   if (mGetTimestampOnly) {
-      mLastTimestamp = mClipTime > mPrimTime ? mClipTime : mPrimTime;
-      return;
+      g_debug("%s: Unable to get mPrimTime.", __FUNCTION__);
    }
 
    /* After got both timestamp, choose latest one as active selection. */
+   mGHSelection = GDK_SELECTION_PRIMARY;
    if (mClipTime > mPrimTime) {
       mGHSelection = GDK_SELECTION_CLIPBOARD;
-      if (mClipTime > 0 && mClipTime == mLastTimestamp) {
-         g_debug("%s: clip is not changed\n", __FUNCTION__);
-         SendClipNotChanged();
-         return;
-      }
-      mLastTimestamp = mClipTime;
-   } else {
-      mGHSelection = GDK_SELECTION_PRIMARY;
-      if (mPrimTime > 0 && mPrimTime == mLastTimestamp) {
-         g_debug("%s: clip is not changed\n", __FUNCTION__);
-         SendClipNotChanged();
-         return;
-      }
-      mLastTimestamp = mPrimTime;
    }
 
    Glib::RefPtr<Gtk::Clipboard> refClipboard;
@@ -716,10 +612,10 @@ again:
    /* First check for URIs. This must always be done first */
    bool haveURIs = false;
    std::string format;
-   if (mCP->CheckCapability(DND_CP_CAP_FILE_CP) && refClipboard->wait_is_target_available(FCP_TARGET_NAME_GNOME_COPIED_FILES)) {
+   if (refClipboard->wait_is_target_available(FCP_TARGET_NAME_GNOME_COPIED_FILES)) {
       format = FCP_TARGET_NAME_GNOME_COPIED_FILES;
       haveURIs = true;
-   } else if (mCP->CheckCapability(DND_CP_CAP_FILE_CP) && refClipboard->wait_is_target_available(FCP_TARGET_NAME_URI_LIST)) {
+   } else if (refClipboard->wait_is_target_available(FCP_TARGET_NAME_URI_LIST)) {
       format = FCP_TARGET_NAME_URI_LIST;
       haveURIs = true;
    }
@@ -734,7 +630,7 @@ again:
    /* Try to get image data from clipboard. */
    Glib::RefPtr<Gdk::Pixbuf> img = refClipboard->wait_for_image();
    gsize bufSize;
-   if (mCP->CheckCapability(DND_CP_CAP_IMAGE_CP) && img) {
+   if (img) {
       gchar *buf = NULL;
 
       img->save_to_buffer(buf, bufSize, Glib::ustring("png"));
@@ -742,12 +638,13 @@ again:
           bufSize <= (int)CPCLIPITEM_MAX_SIZE_V3 &&
           CPClipboard_SetItem(&mClipboard, CPFORMAT_IMG_PNG,
                               buf, bufSize)) {
-         validDataInClip = true;
+         mCP.SetRemoteClipboard(&mClipboard);
          g_debug("%s: Got PNG: %"FMTSZ"u\n", __FUNCTION__, bufSize);
       } else {
          g_debug("%s: Failed to get PNG\n", __FUNCTION__);
       }
       g_free(buf);
+      return;
    }
 
    /* Try to get RTF data from clipboard. */
@@ -763,7 +660,7 @@ again:
       haveRTF = true;
    }
 
-   if (mCP->CheckCapability(DND_CP_CAP_RTF_CP) && haveRTF) {
+   if (haveRTF) {
       /*
        * There is a function for waiting for rtf data, but that was leading
        * to crashes. It's use required we instantiate a class that implements
@@ -786,8 +683,7 @@ again:
    }
 
    /* Try to get Text data from clipboard. */
-   if (mCP->CheckCapability(DND_CP_CAP_PLAIN_TEXT_CP) &&
-       refClipboard->wait_is_text_available()) {
+   if (refClipboard->wait_is_text_available()) {
       g_debug("%s: ask for text\n", __FUNCTION__);
       Glib::ustring str = refClipboard->wait_for_text();
       bufSize = str.bytes();
@@ -806,7 +702,7 @@ again:
       /*
        * RTF or text data (or both) in the clipboard.
        */
-      mCP->DestUISendClip(&mClipboard);
+      mCP.SetRemoteClipboard(&mClipboard);
    } else if (!flipped) {
       /*
        * If we get here, we got nothing (no image, URI, text) so
@@ -818,10 +714,6 @@ again:
                      GDK_SELECTION_CLIPBOARD : GDK_SELECTION_PRIMARY;
       flipped = true;
       goto again;
-   } else {
-      g_debug("%s: got nothing, send empty clip back.\n",
-            __FUNCTION__);
-      mCP->DestUISendClip(&mClipboard);
    }
 }
 
@@ -849,16 +741,10 @@ CopyPasteUIX11::LocalReceivedFileListCB(const Gtk::SelectionData& sd)        //
    g_debug("%s: enter", __FUNCTION__);
    const utf::string target = sd.get_target().c_str();
 
-   if (!mCP->CheckCapability(DND_CP_CAP_FILE_CP)) {
-      /*
-       * Disallowed based on caps settings, return.
-       */
-      return;
-   }
    if (target == FCP_TARGET_NAME_GNOME_COPIED_FILES ||
        target == FCP_TARGET_NAME_URI_LIST) {
       LocalGetSelectionFileList(sd);
-      mCP->DestUISendClip(&mClipboard);
+      mCP.SetRemoteClipboard(&mClipboard);
    }
 }
 
@@ -891,13 +777,6 @@ CopyPasteUIX11::LocalGetFileContentsRequestCB(Gtk::SelectionData& sd, // IN
    utf::string pre;
    utf::string post;
 
-   if (!mCP->CheckCapability(DND_CP_CAP_FILE_CONTENT_CP)) {
-      /*
-       * Disallowed based on caps settings, return.
-       */
-      return;
-   }
-
    sd.set(sd.get_target().c_str(), "");
 
    /* Provide URIs for each path in the guest's file list. */
@@ -993,31 +872,17 @@ CopyPasteUIX11::LocalGetSelectionFileList(const Gtk::SelectionData& sd)      //
    while ((newPath = DnD_UriListGetNextFile(source.c_str(),
                                             &index,
                                             &newPathLen)) != NULL) {
-#if defined(linux)
-      if (DnD_UriIsNonFileSchemes(newPath)) {
-         /* Try to get local file path for non file uri. */
-         GFile *file = g_file_new_for_uri(newPath);
-         free(newPath);
-         if (!file) {
-            g_debug("%s: g_file_new_for_uri failed\n", __FUNCTION__);
-            return;
-         }
-         newPath = g_file_get_path(file);
-         g_object_unref(file);
-         if (!newPath) {
-            g_debug("%s: g_file_get_path failed\n", __FUNCTION__);
-            return;
-         }
-      }
-#endif
 
       /*
        * Parse relative path.
        */
       newRelPath = Str_Strrchr(newPath, DIRSEPC) + 1; // Point to char after '/'
 
-      /* Keep track of how big the fcp files are. */
-      if ((size = File_GetSizeEx(newPath)) >= 0) {
+      /*
+       * XXX For directory, value is -1, so if there is any directory,
+       * total size is not accurate.
+       */
+      if ((size = File_GetSize(newPath)) >= 0) {
          totalSize += size;
       } else {
          g_debug("%s: Unable to get file size for %s\n", __FUNCTION__, newPath);
@@ -1171,7 +1036,7 @@ CopyPasteUIX11::GetRemoteClipboardCB(const CPClipboard *clip) // IN
    void *buf;
    size_t sz;
 
-   TRACE_CALL();
+   g_debug("%s: enter\n", __FUNCTION__);
    if (!clip) {
       g_debug("%s: No clipboard contents.", __FUNCTION__);
       return;
@@ -1181,7 +1046,7 @@ CopyPasteUIX11::GetRemoteClipboardCB(const CPClipboard *clip) // IN
    if (mIsClipboardOwner) {
       refClipboard->clear();
       refPrimary->clear();
-      mIsClipboardOwner = false;
+      mIsClipboardOwner = FALSE;
       g_debug("%s: Cleared local clipboard", __FUNCTION__);
    }
 
@@ -1193,21 +1058,6 @@ CopyPasteUIX11::GetRemoteClipboardCB(const CPClipboard *clip) // IN
        CPClipboard_ItemExists(clip, CPFORMAT_RTF)) {
       std::list<Gtk::TargetEntry> targets;
 
-      /*
-       * rtf should be first in the target list otherwise OpenOffice may not
-       * accept paste.
-       */
-      if (CPClipboard_GetItem(clip, CPFORMAT_RTF, &buf, &sz)) {
-         g_debug("%s: RTF data, size %"FMTSZ"u.\n", __FUNCTION__, sz);
-         Gtk::TargetEntry appRtf(TARGET_NAME_APPLICATION_RTF);
-         Gtk::TargetEntry textRtf(TARGET_NAME_TEXT_RICHTEXT);
-
-         targets.push_back(appRtf);
-         targets.push_back(textRtf);
-         mHGRTFData = std::string((const char *)buf);
-         mIsClipboardOwner = true;
-      }
-
       if (CPClipboard_GetItem(clip, CPFORMAT_TEXT, &buf, &sz)) {
          Gtk::TargetEntry stringText(TARGET_NAME_STRING);
          Gtk::TargetEntry plainText(TARGET_NAME_TEXT_PLAIN);
@@ -1221,7 +1071,20 @@ CopyPasteUIX11::GetRemoteClipboardCB(const CPClipboard *clip) // IN
          targets.push_back(compountText);
          mHGTextData = utf::string(reinterpret_cast<char *>(buf),
                                    STRING_ENCODING_UTF8);
-         mIsClipboardOwner = true;
+
+         mIsClipboardOwner = TRUE;
+      }
+
+      if (CPClipboard_GetItem(clip, CPFORMAT_RTF, &buf, &sz)) {
+         g_debug("%s: RTF data, size %"FMTSZ"u.\n", __FUNCTION__, sz);
+         Gtk::TargetEntry appRtf(TARGET_NAME_APPLICATION_RTF);
+         Gtk::TargetEntry textRtf(TARGET_NAME_TEXT_RICHTEXT);
+
+         targets.push_back(appRtf);
+         targets.push_back(textRtf);
+         mHGRTFData = utf::string((const char *)buf);
+
+         mIsClipboardOwner = TRUE;
       }
 
       refClipboard->set(targets,
@@ -1236,41 +1099,21 @@ CopyPasteUIX11::GetRemoteClipboardCB(const CPClipboard *clip) // IN
    if (CPClipboard_GetItem(clip, CPFORMAT_IMG_PNG, &buf, &sz)) {
       g_debug("%s: PNG data, size %"FMTSZ"u.\n", __FUNCTION__, sz);
       /* Try to load buf into pixbuf, and write to local clipboard. */
-      try {
-         Glib::RefPtr<Gdk::PixbufLoader> loader = Gdk::PixbufLoader::create();
+      Glib::RefPtr<Gdk::PixbufLoader> loader = Gdk::PixbufLoader::create();
+
+      if (loader) {
          loader->write((const guint8 *)buf, sz);
          loader->close();
 
          refClipboard->set_image(loader->get_pixbuf());
          refPrimary->set_image(loader->get_pixbuf());
-
-         /*
-          * Record current clipboard timestamp to prevent unexpected clipboard
-          * exchange.
-          *
-          * XXX We should do this for all formats.
-          */
-         mClipTime = 0;
-         mPrimTime = 0;
-         mGetTimestampOnly = true;
-         refClipboard->request_contents(TARGET_NAME_TIMESTAMP,
-            sigc::mem_fun(this, &CopyPasteUIX11::LocalClipboardTimestampCB));
-      } catch (const Gdk::PixbufError& e) {
-         g_message("%s: caught Gdk::PixbufError %s\n", __FUNCTION__, e.what().c_str());
-      } catch (std::exception& e) {
-         g_message("%s: caught std::exception %s\n", __FUNCTION__, e.what());
-      } catch (...) {
-         g_message("%s: caught unknown exception (typename %s)\n", __FUNCTION__,
-                   __cxxabiv1::__cxa_current_exception_type()->name());
       }
       return;
    }
-
    if (CPClipboard_GetItem(clip, CPFORMAT_FILELIST, &buf, &sz)) {
       g_debug("%s: File data.\n", __FUNCTION__);
       DnDFileList flist;
       flist.FromCPClipboard(buf, sz);
-      mTotalFileSize = flist.GetFileSize();
       mHGFCPData = flist.GetRelPathsStr();
 
       refClipboard->set(mListTargets,
@@ -1280,9 +1123,9 @@ CopyPasteUIX11::GetRemoteClipboardCB(const CPClipboard *clip) // IN
                       sigc::mem_fun(this, &CopyPasteUIX11::LocalGetFileRequestCB),
                       sigc::mem_fun(this, &CopyPasteUIX11::LocalClearClipboardCB));
 
-      mIsClipboardOwner = true;
+      mIsClipboardOwner = TRUE;
       mHGGetListTime = GetCurrentTime();
-      mHGGetFileStatus = DND_FILE_TRANSFER_NOT_STARTED;
+      mHGGetFilesInitiated = false;
       mHGCopiedUriList = "";
    }
 
@@ -1295,7 +1138,7 @@ CopyPasteUIX11::GetRemoteClipboardCB(const CPClipboard *clip) // IN
          refPrimary->set(mListTargets,
                          sigc::mem_fun(this, &CopyPasteUIX11::LocalGetFileContentsRequestCB),
                          sigc::mem_fun(this, &CopyPasteUIX11::LocalClearClipboardCB));
-         mIsClipboardOwner = true;
+         mIsClipboardOwner = TRUE;
       }
    }
 }
@@ -1328,7 +1171,7 @@ CopyPasteUIX11::LocalPrepareFileContents(const CPClipboard *clip) // IN
    CPFileContentsList *contentsList = NULL;
    size_t nFiles = 0;
    CPFileItem *fileItem = NULL;
-   char *tempDir = NULL;
+   Unicode tempDir = NULL;
    size_t i = 0;
    bool ret = false;
 
@@ -1522,14 +1365,11 @@ CopyPasteUIX11::GetLocalFilesDone(bool success)
 
    if (mBlockAdded) {
       g_debug("%s: removing block for %s\n", __FUNCTION__, mHGStagingDir.c_str());
-      /* We need to make sure block subsystem has not been shut off. */
-      if (DnD_BlockIsReady(mBlockCtrl)) {
-         mBlockCtrl->RemoveBlock(mBlockCtrl->fd, mHGStagingDir.c_str());
-      }
+      mBlockCtrl->RemoveBlock(mBlockCtrl->fd, mHGStagingDir.c_str());
       mBlockAdded = false;
    }
 
-   mHGGetFileStatus = DND_FILE_TRANSFER_FINISHED;
+   mFileTransferDone = true;
    if (success) {
       /*
        * Mark current staging dir to be deleted on next reboot for FCP. The
@@ -1541,35 +1381,7 @@ CopyPasteUIX11::GetLocalFilesDone(bool success)
       /* Copied files are already removed in common layer. */
       mHGStagingDir.clear();
    }
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * CopyPasteUIX11::SendClipNotChanged --
- *
- *    Send a not-changed clip to host.
- *
- * Results:
- *    None.
- *
- * Side effects:
- *    None.
- *
- *-----------------------------------------------------------------------------
- */
-
-void
-CopyPasteUIX11::SendClipNotChanged(void)
-{
-   CPClipboard clip;
-
-   g_debug("%s: enter.\n", __FUNCTION__);
-   CPClipboard_Init(&clip);
-   CPClipboard_SetChanged(&clip, FALSE);
-   mCP->DestUISendClip(&clip);
-   CPClipboard_Destroy(&clip);
+   mHGGetFilesInitiated = false;
 }
 
 
@@ -1591,6 +1403,6 @@ CopyPasteUIX11::SendClipNotChanged(void)
 void
 CopyPasteUIX11::Reset(void)
 {
-   TRACE_CALL();
+   g_debug("%s: enter\n", __FUNCTION__);
    /* Cancel any pending file transfer. */
 }
