@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2009-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2009 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -44,19 +44,9 @@ extern "C" {
 
 #include "unicodeOperations.h"
 
-#include "guestCopyPaste.hh"
+#include "copyPaste.hh"
 
-/*
- * Make sure exception types are public and therefore shared between libg*mm
- * and this plugin. 
- *
- * See
- * http://gcc.gnu.org/wiki/Visibility#Problems_with_C.2B-.2B-_exceptions_.28please_read.21.29
- */
-#pragma GCC visibility push(default)
 #include <gtkmm.h>
-#pragma GCC visibility pop
-
 #include <list>
 #include <vector>
 
@@ -65,6 +55,10 @@ extern "C" {
 #include <gdk/gdkx.h>
 #include "vmware/guestrpc/tclodefs.h"
 
+extern "C" {
+void CopyPasteSelectionReceivedCB(GtkWidget *w, GtkSelectionData *sel_data, void *data);
+}
+
 class CopyPasteUIX11 : public sigc::trackable
 {
 public:
@@ -74,7 +68,7 @@ public:
    void VmxCopyPasteVersionChanged(RpcChannel *chan,
                                    uint32 version);
    void SetCopyPasteAllowed(bool isCopyPasteAllowed)
-   { mCP->SetCopyPasteAllowed(isCopyPasteAllowed); }
+      { mCP.SetCopyPasteAllowed(isCopyPasteAllowed); }
    void Reset(void);
    void SetBlockControl(DnDBlockControl *blockCtrl)
       { Debug("Setting mBlockCtrl to %p\n", blockCtrl);
@@ -92,12 +86,11 @@ private:
    void LocalClearClipboardCB(void);
 
    /* gh */
-   void GetLocalClipboard(void);
+   bool GetLocalClipboard(CPClipboard *clip);
    void LocalClipboardTimestampCB(const Gtk::SelectionData& sd);
    void LocalPrimTimestampCB(const Gtk::SelectionData& sd);
    void LocalReceivedFileListCB(const Gtk::SelectionData& selection_data);
    void GetLocalFilesDone(bool success);
-   void SendClipNotChanged(void);
 
    /* Conversion methods. */
    utf::utf8string GetNextPath(utf::utf8string &str, size_t& index);
@@ -107,30 +100,28 @@ private:
    VmTimeType GetCurrentTime(void);
 
    // Member variables
-   GuestCopyPasteMgr *mCP;
+   CopyPaste mCP;
    bool mClipboardEmpty;
    utf::string mHGStagingDir;
    std::list<Gtk::TargetEntry> mListTargets;
    bool mIsClipboardOwner;
    uint64 mClipTime;
    uint64 mPrimTime;
-   uint64 mLastTimestamp;
    GdkAtom mGHSelection;
    CPClipboard mClipboard;
 
    /* File vars. */
+   bool mHGGetFilesInitiated;
    VmTimeType mHGGetListTime;
    utf::string mHGCopiedUriList;
    utf::utf8string mHGFCPData;
    utf::string mHGTextData;
-   std::string mHGRTFData;
+   utf::string mHGRTFData;
    std::vector<utf::string> mHGFileContentsList;
-   DND_FILE_TRANSFER_STATUS mHGGetFileStatus;
+   bool mFileTransferDone;
    bool mBlockAdded;
    DnDBlockControl *mBlockCtrl;
    bool mInited;
-   uint64 mTotalFileSize;
-   bool mGetTimestampOnly;
 };
 
 #endif // __COPYPASTE_UI_X11_H__
