@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2007-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2007 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -127,12 +127,9 @@ Bool CPClipboard_GetItem(const CPClipboard *clip, DND_CPFORMAT fmt,
                          void **buf, size_t *size);
 Bool CPClipboard_ItemExists(const CPClipboard *clip, DND_CPFORMAT fmt);
 Bool CPClipboard_IsEmpty(const CPClipboard *clip);
-#if !defined(SWIG)
 size_t CPClipboard_GetTotalSize(const CPClipboard *clip);
-#endif
 Bool CPClipboard_Copy(CPClipboard *dest, const CPClipboard *src);
 Bool CPClipboard_Serialize(const CPClipboard *clip, DynBuf *buf);
-Bool CPClipboard_Unserialize(CPClipboard *clip, const void *buf, size_t len);
-Bool CPClipboard_Strip(CPClipboard *clip, uint32 caps);
+Bool CPClipboard_Unserialize(CPClipboard *clip, void *buf, size_t len);
 
 #endif // _DND_CLIPBOARD_H_
