@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2010 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -27,6 +27,8 @@
 #define __DNDPLUGIN_INT_H__
 
 extern "C" {
+   #include "guestApp.h"
+   #include "eventManager.h"
    #include "conf.h"
 }
 
@@ -34,9 +36,8 @@ extern "C" {
 #define RPC_POLL_TIME            10
 #define POINTER_POLL_TIME        10
 #define UNGRABBED_POS (-100)
-#define VMWARE_CLIP_FORMAT_NAME     L"VMwareClipFormat"
+#define VMWARE_CLIP_FORMAT_NAME  L"VMwareClipFormat"
 #define TOOLS_DND_VERSION_3         "tools.capability.dnd_version 3"
-#define TOOLS_DND_VERSION_4         "tools.capability.dnd_version 4"
 #define QUERY_VMX_DND_VERSION       "vmx.capability.dnd_version"
 #define TOOLS_COPYPASTE_VERSION     "tools.capability.copypaste_version"
 #define QUERY_VMX_COPYPASTE_VERSION "vmx.capability.copypaste_version"
