@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2010 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -58,7 +58,6 @@ DnDCPShutdown(gpointer src,
       p->UnregisterCP();
       p->UnregisterDnD();
    }
-   CopyPasteDnDWrapper::Destroy();
 }
 
 
@@ -174,33 +173,29 @@ ToolsOnLoad(ToolsAppCtx *ctx)
       NULL
    };
 
-   if (ctx->rpc != NULL) {
-      ToolsPluginSignalCb sigs[] = {
-         { TOOLS_CORE_SIG_CAPABILITIES, (void *) DnDCPCapabilities, NULL },
-         { TOOLS_CORE_SIG_RESET, (void *) DnDCPReset, NULL },
-         { TOOLS_CORE_SIG_SET_OPTION, (void *) DnDCPSetOption, NULL },
-         { TOOLS_CORE_SIG_SHUTDOWN, (void *) DnDCPShutdown, NULL }
-      };
+   ToolsPluginSignalCb sigs[] = {
+      { TOOLS_CORE_SIG_CAPABILITIES, (void *) DnDCPCapabilities, NULL },
+      { TOOLS_CORE_SIG_RESET, (void *) DnDCPReset, NULL },
+      { TOOLS_CORE_SIG_SET_OPTION, (void *) DnDCPSetOption, NULL },
+      { TOOLS_CORE_SIG_SHUTDOWN, (void *) DnDCPShutdown, NULL }
+   };
 
-      ToolsAppReg regs[] = {
-         { TOOLS_APP_SIGNALS, VMTools_WrapArray(sigs, sizeof *sigs, ARRAYSIZE(sigs)) }
-      };
+   ToolsAppReg regs[] = {
+      { TOOLS_APP_SIGNALS, VMTools_WrapArray(sigs, sizeof *sigs, ARRAYSIZE(sigs)) }
+   };
 
-      /*
-       * DnD/CP Initialization here.
-       */
+   /*
+    * DnD/CP Initialization here.
+    */
 
-      CopyPasteDnDWrapper *p = CopyPasteDnDWrapper::GetInstance();
-      if (p) {
-         p->Init(ctx);
-         p->PointerInit();
-      }
-
-      regData.regs = VMTools_WrapArray(regs, sizeof *regs, ARRAYSIZE(regs));
-      return &regData;
+   CopyPasteDnDWrapper *p = CopyPasteDnDWrapper::GetInstance();
+   if (p) {
+      p->Init(ctx);
+      p->PointerInit();
    }
 
-   return NULL;
+   regData.regs = VMTools_WrapArray(regs, sizeof *regs, ARRAYSIZE(regs));
+   return &regData;
 }
 
 }
