@@ -51,6 +51,7 @@
 
 #include "vmware.h"
 #include "vmblock.h"
+#include "vmsignal.h"
 #include "wrapper.h"
 
 #include "wrapper_version.h"
@@ -70,6 +71,7 @@ static Bool MakeDirectory(const char *path, mode_t mode, uid_t uid, gid_t gid);
 static Bool ChmodChownDirectory(const char *path,
                                 mode_t mode, uid_t uid, gid_t gid);
 #endif
+static void MaskSignals(void);
 static Bool StartVMwareUser(char *const envp[]);
 
 
@@ -106,6 +108,8 @@ main(int argc,
      char *argv[],
      char *envp[])
 {
+   MaskSignals();
+
 #ifdef TOGGLE_VMBLOCK
    ToggleVMBlock();
 #endif
@@ -361,6 +365,42 @@ out:
 #endif  // ifdef TOGGLE_VMBLOCK
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MaskSignals --
+ *
+ *      Sets SIG_IGN as the handler for SIGUSR1 and SIGUSR2 which may arrive
+ *      prematurely from our services script.  See bug 542135.
+ *
+ * Results:
+ *      Returns if applicable signals are blocked.
+ *      Exits with EXIT_FAILURE otherwise.
+ *
+ * Side effects:
+ *      SIG_IGN disposition persists across execve().  These signals will
+ *      remain masked until vmware-user defines its own handlers.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static void
+MaskSignals(void)
+{
+   int const signals[] = {
+      SIGUSR1,
+      SIGUSR2
+   };
+   struct sigaction olds[ARRAYSIZE(signals)];
+
+   if (Signal_SetGroupHandler(signals, olds, ARRAYSIZE(signals),
+                              SIG_IGN) == 0) {
+      /* Signal_SetGroupHandler will write error message to stderr. */
+      exit(EXIT_FAILURE);
+   }
+}
+
+
 /*
  *----------------------------------------------------------------------------
  *
@@ -390,7 +430,8 @@ StartVMwareUser(char *const envp[])
    int fd = -1;
    int ret;
    char path[MAXPATHLEN];
-   char *argv[4];
+   char *argv[6];
+   size_t idx = 0;
 
    if (!BuildExecPath(path, sizeof path)) {
       return FALSE;
@@ -440,25 +481,25 @@ StartVMwareUser(char *const envp[])
     * can't parse the descriptor to pass as an argument.  We set up the
     * argument vector accordingly.
     */
-   argv[0] = path;
+   argv[idx++] = path;
+   argv[idx++] = "-n";
+   argv[idx++] = "vmusr";
 
    if (fd < 0) {
       Error("could not open %s\n", VMBLOCK_DEVICE);
-      argv[1] = NULL;
    } else {
       char fdStr[8];
 
       ret = snprintf(fdStr, sizeof fdStr, "%d", fd);
       if (ret == 0 || ret >= sizeof fdStr) {
          Error("could not parse file descriptor (%d)\n", fd);
-         argv[1] = NULL;
       } else {
-         argv[1] = "--blockFd";
-         argv[2] = fdStr;
-         argv[3] = NULL;
+         argv[idx++] = "--blockFd";
+         argv[idx++] = fdStr;
       }
    }
 
+   argv[idx++] = NULL;
    CompatExec(path, argv, envp);
 
    /*
@@ -466,7 +507,7 @@ StartVMwareUser(char *const envp[])
     * if CompatExec fails.
     */
    Error("could not execute %s: %s\n", path, strerror(errno));
-   exit(EXIT_FAILURE);
+   _exit(EXIT_FAILURE);
 }
 
 
@@ -493,10 +534,10 @@ Bool
 BuildExecPath(char *execPath,           // OUT: Buffer to store executable's path
               size_t execPathSize)      // IN : size of execPath buffer
 {
-   if (execPathSize < sizeof VMWARE_USER_PATH) {
+   if (execPathSize < sizeof VMTOOLSD_PATH) {
       return FALSE;
    }
-   strcpy(execPath, VMWARE_USER_PATH);
+   strcpy(execPath, VMTOOLSD_PATH);
    return TRUE;
 }
 #endif // ifndef USES_LOCATIONS_DB
